import requests
import os
from pyVmomi import vim
import atexit
import json
import pprint
import redis
import time


from pyVim.connect import SmartConnect, Disconnect
r = redis.Redis(host='localhost', port=6379, db=0)

data = {}


def connect(env):
    try:
        if env['IGN8_VMWARE_SSL'] == "disable":
            service_instance = SmartConnect(host=env['IGN8_VMWARE_URL'] ,
                                            user=env['IGN8_VMWARE_USERNAME'],
                                            pwd=env['IGN8_VMWARE_PASSWORD'],
                                            disableSslCertValidation=True)
        else:
            service_instance = SmartConnect(host=env['IGN8_VMWARE_URL'] ,
                                            user=env['IGN8_VMWARE_USERNAME'],
                                            pwd=env['IGN8_VMWARE_PASSWORD'])
        #atexit.register(Disconnect, service_instance)
    except IOError as io_error:
        print(io_error)
    if not service_instance:
        raise SystemExit("Unable to connect to host with supplied credentials.")
    return service_instance


def  get_env():
  myenv = {}
  myenv['IGN8_VMWARE_URL'] = os.getenv("IGN8_VMWARE_URL")
  myenv['IGN8_VMWARE_USERNAME'] = os.getenv("IGN8_VMWARE_USERNAME")
  myenv['IGN8_VMWARE_PASSWORD'] = os.getenv("IGN8_VMWARE_PASSWORD")
  myenv['IGN8_VMWARE_SSL'] = os.getenv("IGN8_VMWARE_SSL")
  return myenv


# Function to get a list of VMs

def get_cluster_list(content):
    limit = 20
    cluster_list = []
    cluster_details = {}
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [vim.ClusterComputeResource], True
    )
    # promnt items in container
    pprint.pprint(container.view)

    for cluster in container.view:
        if limit == 0:
            break
        limit -= 1
        cluster_list.append(cluster.name)
        cluster_details[cluster.name] = {}
        cluster_details[cluster.name]['configuration'] = cluster.configuration
        cluster_details[cluster.name]['datastore'] = cluster.datastore
        cluster_details[cluster.name]['host'] = cluster.host
        cluster_details[cluster.name]['network'] = cluster.network
        cluster_details[cluster.name]['summary'] = cluster.summary
    return cluster_list, cluster_details

def get_vm_details(content, vmname):
    vm_details = {}
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [vim.VirtualMachine], True
    )
    for vm in container.view:
        if vm.name == vmname:
            vm_details[vm.name] = {}
            vm_details[vm.name]['config'] = vm.summary.config
            vm_details[vm.name]['guest'] = vm.summary.guest
            vm_details[vm.name]['runtime'] = vm.summary.runtime
            vm_details[vm.name]['storage'] = vm.summary.storage
            vm_details[vm.name]['summary'] = vm.summary
            return vm_details

def get_vm_list(content):
    vm_list = []
    vm_details = {}
    container = content.viewManager.CreateContainerView(
        content.rootFolder, [vim.VirtualMachine], True
    )
    for vm in container.view:
        vm_list.append(vm.name)
        vm_details[vm.name] = {}
        #vm_details[vm.name]['config'] = vm.summary.config
        #vm_details[vm.name]['guest'] = vm.summary.guest
        #vm_details[vm.name]['runtime'] = vm.summary.runtime
        #vm_details[vm.name]['storage'] = vm.summary.storage
        #vm_details[vm.name]['summary'] = vm.summary
    return vm_list, vm_details

def vm2dict(datacenter, cluster, host, vm, summary):
    # If nested folder path is required, split into a separate function
    vmname = vm.summary.config.name
    data[datacenter][cluster][host][vmname]['folder'] = vm.parent.name
    data[datacenter][cluster][host][vmname]['mem'] = summary['mem']
    data[datacenter][cluster][host][vmname]['diskGB'] = summary['diskGB']
    data[datacenter][cluster][host][vmname]['cpu'] = summary['cpu']
    data[datacenter][cluster][host][vmname]['path'] = summary['path']
    data[datacenter][cluster][host][vmname]['net'] = summary['net']
    data[datacenter][cluster][host][vmname]['ostype'] = summary['ostype']
    data[datacenter][cluster][host][vmname]['state'] = summary['state']
    data[datacenter][cluster][host][vmname]['annotation'] = summary['annotation']


def data2json(raw_data, args):
    with open(args.jsonfile, 'w') as json_file:
        json.dump(raw_data, json_file)    

def get_nics(guest):
    nics = {}
    for nic in guest.net:
        if nic.network:  # Only return adapter backed interfaces
            if nic.ipConfig is not None and nic.ipConfig.ipAddress is not None:
                nics[nic.macAddress] = {}  # Use mac as uniq ID for nic
                nics[nic.macAddress]['netlabel'] = nic.network
                ipconf = nic.ipConfig.ipAddress
                i = 0
                nics[nic.macAddress]['ipv4'] = {}
                for ip in ipconf:
                    if ":" not in ip.ipAddress:  # Only grab ipv4 addresses
                        nics[nic.macAddress]['ipv4'][i] = ip.ipAddress
                        nics[nic.macAddress]['prefix'] = ip.prefixLength
                        nics[nic.macAddress]['connected'] = nic.connected
                    i = i+1
    return nics


def vmsummary(summary, guest):
    vmsum = {}
    config = summary.config
    net = get_nics(guest)
    vmsum['mem'] = str(config.memorySizeMB / 1024)
    vmsum['diskGB'] = str("%.2f" % (summary.storage.committed / 1024**3))
    vmsum['cpu'] = str(config.numCpu)
    vmsum['path'] = config.vmPathName
    vmsum['ostype'] = config.guestFullName
    vmsum['state'] = summary.runtime.powerState
    vmsum['annotation'] = config.annotation if config.annotation else ''
    vmsum['net'] = net

    return vmsum



# Connect to the vCenter server
def connect_to_vcenter():
    myenv = get_env()
    service_instance = connect(myenv)
    if not service_instance:
        print("Unable to connect to vCenter server.")
        return None
    return service_instance.RetrieveContent()

def get_obj(content, vimtype, name = None):
    return [item for item in content.viewManager.CreateContainerView(
        content.rootFolder, [vimtype], recursive=True).view]

content = connect_to_vcenter()
vm_list, vm_details = get_vm_list(content)

for vm in vm_list:
    print(vm)
    detailkey = "ign8:vmware:" + vm + ":details"
    knownkey = "ign8:vmware:" + vm + ":known"
    timestamp = time.time()
    get = r.get(detailkey)
    if get is None:
        print("not in redis")
        vm_details = get_vm_details(content, vm)
        r.set(detailkey, str(vm_details), ex=3600)
        r.set(knownkey, str(timestamp))
    else:
        print("in redis")
        r.set(knownkey, str(timestamp))
        print(get)







#       hosts = cluster.host  # Variable to make pep8 compliance
#        for host in hosts:  # Iterate through Hosts in the Cluster
#            hostname = host.summary.config.name
#            # Add VMs to data dict by config name
#            data[datacenter.name][cluster.name][hostname] = {}
#            vms = host.vm
#            for vm in vms:  # Iterate through each VM on the host
#                vmname = vm.summary.config.name
#                data[datacenter.name][cluster.name][hostname][vmname] = {}
#                summary = vmsummary(vm.summary, vm.guest)
#                vm2dict(datacenter.name, cluster.name, hostname, vm, summary)





  


from django.shortcuts import render
from .models import Selinux
from rest_framework import viewsets
from .models import SElinuxEvent
from .serializers import SElinuxEventSerializer, SelinuxDataSerializer


from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.views import View
import json
import pprint
from .models import Selinux, SElinuxEvent


def selinux_list(request):
    selinux_entries = Selinux.objects.all()
    return render(request, 'selinux_list.html', {'selinux_entries': selinux_entries})



@method_decorator(csrf_exempt, name='dispatch')
class UploadSelinuxDataView(View):
    def post(self, request, *args, **kwargs):
        try:
            data = json.loads(request.body.decode('utf-8'))

            # Assuming 'hostname' is a unique key
            hostname = data.get('hostname')

            selinux_instance, created = Selinux.objects.update_or_create(
                hostname=hostname,
                defaults=data
            )

            if created:
                return JsonResponse({'message': f'Data for {hostname} created successfully.'}, status=201)
            else:
                return JsonResponse({'message': f'Data for {hostname} updated successfully.'}, status=200)

        except json.JSONDecodeError:
            return JsonResponse({'error': 'Invalid JSON data.'}, status=400)
    
def selinux_event_list(request):
    selinux_event_entries = SElinuxEvent.objects.all()
    pprint.pprint(selinux_event_entries)

    return render(request, 'selinux_event_list.html', {'selinux_event_entries': selinux_event_entries})


@method_decorator(csrf_exempt, name='dispatch')
class UploadSElinuxEventView(View):
    def post(self, request, *args, **kwargs):
        try:
            data = json.loads(request.body.decode('utf-8'))
            digest = data.get('digest')

            SElinuxEvent_instance, created = SElinuxEvent.objects.update_or_create(
                defaults=data
            )

            if created:
                return JsonResponse({'message': f'Data for {digest} created successfully.'}, status=201)
            else:
                return JsonResponse({'message': f'Data for {digest} updated successfully.'}, status=200)

        except json.JSONDecodeError:
            return JsonResponse({'error': 'Invalid JSON data.'}, status=400)
        




# Written by Chris Arceneaux
# GitHub: https://github.com/carceneaux
# Email: carceneaux@thinksis.com
# Website: http://arsano.ninja
#
# This code has been released under the terms of the Apache-2.0 license
# http://opensource.org/licenses/Apache-2.0

"""
This module implements simple helper functions for working with:

- Virtual Machine Disks
- First Class Disks
"""


def retrieve_fcd(content, datastore, vdisk):
    """
    Retrieves the managed object for the first class disk specified

    Sample Usage:

    retrieve_fcd(content, datastore, "First Class Disk Name")
    """
    # Set vStorageObjectManager
    storage = content.vStorageObjectManager

    # Retrieve First Class Disks
    fcd = None
    for disk in storage.ListVStorageObject(datastore):
        disk_info = storage.RetrieveVStorageObject(disk, datastore)
        if disk_info.config.name == vdisk:
            fcd = disk_info
            break
    if not fcd:
        raise RuntimeError("First Class Disk not found.")
    return fcd


def retrieve_fcd_snapshot(content, datastore, vdisk, snapshot_name):
    """
    Retrieves the managed object for the fcd snapshot specified

    Sample Usage:

    retrieve_fcd_snapshot(content, datastore, vdisk, "Snapshot Name")
    """
    # Set vStorageObjectManager
    storage = content.vStorageObjectManager

    # Retrieve Snapshot
    fcd_snapshot = None
    snaps = storage.RetrieveSnapshotInfo(vdisk.config.id, datastore)
    for snapshot in snaps.snapshots:
        if snapshot.description == snapshot_name:
            fcd_snapshot = snapshot.id
            break
    if not fcd_snapshot:
        raise RuntimeError("Snapshot not found.")
    return fcd_snapshot



from django.db import models

class Selinux(models.Model):
    hostname = models.CharField(max_length=128, primary_key=True)
    status = models.CharField(max_length=50)
    mount = models.CharField(max_length=50)
    rootdir = models.CharField(max_length=50)
    policyname = models.CharField(max_length=50)
    current_mode = models.CharField(max_length=50)
    configured_mode = models.CharField(max_length=50)
    mslstatus = models.CharField(max_length=50)
    memprotect = models.CharField(max_length=50)
    maxkernel = models.CharField(max_length=50)
    total = models.CharField(max_length=50)
    success = models.CharField(max_length=50)
    failed = models.CharField(max_length=50)
    sealerts = models.CharField(max_length=50)
    def __str__(self):
        return self.hostname
    
    class Meta:
        db_table = 'selinux'
        verbose_name = 'Selinux'
        verbose_name_plural = 'Selinux'
        ordering = ['hostname']
# the id must auto increment, otherwise the data will be overwritten


class SElinuxEvent(models.Model):
    digest = models.CharField(max_length=256, primary_key=True)
    hostname = models.CharField(max_length=128)
    event = models.CharField(max_length=1024)
    date = models.DateField()
    time = models.TimeField()
    serial_num = models.IntegerField()
    event_kind = models.CharField(max_length=256, blank=True, null=True)
    session = models.CharField(max_length=256, blank=True, null=True)
    subj_prime = models.CharField(max_length=256, blank=True, null=True)
    subj_sec = models.CharField(max_length=256, blank=True, null=True)
    subj_kind = models.CharField(max_length=256, blank=True, null=True)
    action = models.CharField(max_length=256, blank=True, null=True)
    result = models.CharField(max_length=256, blank=True, null=True)
    obj_prime = models.CharField(max_length=256, blank=True, null=True)
    obj_sec = models.CharField(max_length=256, blank=True, null=True)
    obj_kind = models.CharField(max_length=256, blank=True, null=True)
    how = models.CharField(max_length=256, blank=True, null=True)

    def __str__(self):
        return self.digest
    
    class Meta:
        db_table = 'selinux_event'
        verbose_name = 'SElinuxEvent'
        verbose_name_plural = 'SElinuxEvent'
        ordering = ['date', 'time', 'hostname']



import os
import sys
from ..common import prettyllog




def main():
    prettyllog("ui", "ui", "ui", "new", "000", "ui")
    ign8_ui_port  = os.environ.get("IGN8_UI_PORT", "8000")
    ign8_ui_host = os.environ.get("IGN8_UI_HOST", "ign8.openknowit.com")
    ign8_ui_debug = os.environ.get("IGN8_UI_DEBUG", "True")
    ign8_


                          



import requests
import os
from ..common import prettyllog

import pprint
import json


debug = True





import os
import requests

def login():
    baseurl = os.getenv('IGN8_SEMAPHORE_URL')
    user = os.getenv('IGN8_SEMAPHORE_USER')
    password = os.getenv('IGN8_SEMAPHORE_PASSWORD')
    url = f"{baseurl}/api/auth/login"
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }
    data = {
        'auth': user,
        'password': password
    }

    # Create a session
    session = requests.Session()

    # Make the login request
    response = session.post(url, headers=headers, json=data)
    if response.status_code == 204:
        # Successful request
        prettyllog("semaphore", "Init", "login", "ok",  response.status_code , "login successful", severity="INFO")
        return session  # Return the session for subsequent requests
    else:
        # ERRORed request
        prettyllog("semaphore", "Init", "login", "error", response.status_code , "login failed", severity="ERROR")
        return None
    
def create_project(session, project):
    baseurl = os.getenv('IGN8_SEMAPHORE_URL')
    project_url = f"{baseurl}/api/projects"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.post(project_url, headers=headers, json=project)
    if response.status_code == 201:
        # Successful request
        prettyllog("semaphore", "create", project['name'], "ok", response.status_code , "create project", severity="INFO")
        return response.json()
    else:
        # Failed request
        prettyllog("semaphore", "create", project['name'], "error", response.status_code , "create project", severity="ERROR")


def get_project(session):
    baseurl = os.getenv('IGN8_SEMAPHORE_URL')
    project_url = f"{baseurl}/api/projects"  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.get(project_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        projects = response.json()
        # map prokects by name
        projects_by_name = {}
        for project in projects:
            projects_by_name[project['name']] = project
            if debug:
                prettyllog("semaphore", "get", project['name'], "ok", response.status_code , "loadning projects", severity="DEBUG")
        prettyllog("semaphore", "get", "project", "ok", response.status_code , "loadning projects", severity="INFO")
        return projects_by_name
    else:
        # Failed request
        prettyllog("semaphore", "get", "project", "error", response.status_code , "loadning projects", severity="ERROR")

def get_inventory(session, project_id):
    baseurl = os.getenv('IGN8_SEMAPHORE_URL')
    inventory_url = f"{baseurl}/api/project/{project_id}/inventory?sort=name&order=asc' "  # Adjust the URL as needed
    headers = {
        'accept': 'application/json',
        'Content-Type': 'application/json'
    }

    # Use the session for the request
    response = session.get(inventory_url, headers=headers)
    if response.status_code == 200:
        # Successful request
        inventory = response.json()
        # map prokects by name
        inventory_by_name = {}
        for item in inventory:
            inventory_by_name[item['name']] = item
            if debug:
                prettyllog("semaphore", "get", item['name'], "ok", response.status_code , "loadning inventory", severity="DEBUG")
        prettyllog("semaphore", "get", "inventory", "ok", response.status_code , "loadning inventory", severity="INFO")
        return inventory_by_name
    else:
        # Failed request
        prettyllog("semaphore", "get", "inventory", "error", response.status_code , "loadning inventory", severity="ERROR")

def check_env():
    envok = True
    myenv = {}
    known_git_types = ['github', 'gitlab', 'bitbucket', 'gitea', 'gogs']
    if (os.getenv('IGN8_SEMAPHORE_URL') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "IGN8_SEMAPHORE_URL not set", severity="ERROR")
        envok = False
    else:
        myenv['IGN8_SEMAPHORE_URL'] = os.getenv('IGN8_SEMAPHORE_URL')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "IGN8_SEMAPHORE_URL set", severity="INFO")

    if (os.getenv('IGN8_SEMAPHORE_USER') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "IGN8_SEMAPHORE_USER not set", severity="ERROR")
        envok = False
    else:       
        myenv['IGN8_SEMAPHORE_USER'] = os.getenv('IGN8_SEMAPHORE_USER')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "IGN8_SEMAPHORE_USER set", severity="INFO")

    if (os.getenv('IGN8_SEMAPHORE_PASSWORD') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "IGN8_SEMAPHORE_PASSWORD not set", severity="ERROR")
        envok = False
    else:
        myenv['IGN8_SEMAPHORE_PASSWORD'] = os.getenv('IGN8_SEMAPHORE_PASSWORD')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "IGN8_SEMAPHORE_PASSWORD set", severity="INFO")

    if (os.getenv('IGN8_GIT_TYPE') == None):
        prettyllog("semaphore", "Init", "env", "error", 1 , "IGN8_GIT_TYPE not set", severity="ERROR")
        envok = False
    elif (os.getenv('IGN8_GIT_TYPE') not in known_git_types):
        prettyllog("semaphore", "Init", "env", "error", 1 , "IGN8_GIT_TYPE not set %s " % known_git_types, severity="ERROR")
        envok = False
    else:
        myenv['IGN8_GIT_TYPE'] = os.getenv('IGN8_GIT_TYPE')
        prettyllog("semaphore", "Init", "env", "ok", 0 , "IGN8_GIT_TYPE set", severity="INFO")
    return envok, myenv



def read_config():
    f = open("etc/ign8/ign8.json", "r")
    mainconf = json.load(f)
    prettyllog("semaphore", "Init", "main","main configuration", 0 , "Getting main config", severity="INFO")
    f.close()
    subconf = {}

    # check if automation is enabled for semaphore
    for subproject in mainconf['subprojects']:
        if subproject['engine'] == 'semaphore':
            subconf[subproject['name']] = subproject
            prettyllog("semaphore", "Init", "subpropject", subproject['name'] , "000", "Getting subproject config", severity="DEBUG")
    return True, mainconf, subconf



    


    


 # Example: Get a list of your projects

def check_project(projectname, env):
    # we need to check if the project exists in git
    # if not we need to create it
    # if it exists we need to check if it exists in semaphore
    # if not we need to create it
    if env['IGN8_GIT_TYPE'] == 'gitea':
        from ..gitea.git import get_git_projects
        git_projects = get_git_projects()
        git_project_names = []
        for git_project in git_projects:
            git_project_names.append(git_project['name'])
        if projectname in git_project_names:
            prettyllog("semaphore", "Check", projectname, "ok", 0 , "project exists in git", severity="INFO")
            # check if project exists in semaphore
            # if not create it
            # if it exists check if it has the correct settings
            # if not update it
        else:
            prettyllog("semaphore", "Create", projectname, "missing", 1 , "project missing in git", severity="WARNING")
            from ..gitea.git import create_git_project 
            project = {}
            project['name'] = projectname
            project['description'] = "ign8 project"
            project['private'] = True
            project['auto_init'] = True

            # create project in git
            create_git_project(project)
            # create project in semaphore

            # create project in git
            # create project in semaphore
            # create project in awx
            # create project in vault
            # create project in dns
    else:
        prettyllog("project", "semaphore", projectname, "error", 1 , "git type not supported", severity="ERROR")
        exit(1)



    



    




def main():
    semaphore, mainconf, subprojects = read_config()
    organization = mainconf['organisation']['name']
    mysubprojects = mainconf['subprojects']
    checkenv, env = check_env()
    if not checkenv:
        prettyllog("semaphore", "main", "config", "error", 1 , "No semaphore config found", severity="ERROR")
        exit(1)

    state = {}
    session = login()
    # Check if the main project exists , it is the organization name
    # if not create it
    # if it exists check if it has the correct settings
    # if not update it
    prettyllog("semaphore", "main", "config", organization, 1 , "Check the main project", severity="INFO")
    check_project(organization, env)




    if session:
        projects = get_project(session)
        for project in projects:
            projectname = projects[project]['name']
            state[projectname] = {}
            state[projectname]['project'] = projects[project]
            state[projectname]['inventory'] = {}
            inventory = get_inventory(session, projects[project]['id'])
            for item in inventory:
                itemname = inventory[item]['name']
                state[projectname]['inventory'][itemname] = {}
                state[projectname]['inventory'][itemname]['item'] = inventory[item]
                prettyllog("semaphore", "main", item, "ok", 0 , "item", severity="INFO")

    missings = {}
    for subproject in subprojects:
        if subproject in state:
            prettyllog("semaphore", "main", subproject, "ok", 0 , "subproject", severity="INFO")
        else:
            prettyllog("semaphore", "main", subproject, "missing", 1 , "subproject", severity="DEBUG")
            missings[subproject] = subprojects[subproject]

    orphans = {}    
    for state_project in state:
        if state_project in subprojects or state_project == organization:
            prettyllog("semaphore", "check project", state_project, "ok", 0 , "subproject", severity="INFO")
        else:
            prettyllog("semaphore", "check project", state_project, "orphan", 1 , "subproject", severity="WARNING")
            orphans[state_project] = state[state_project]


    for missing in missings:
        prettyllog("semaphore", "createi project", missing, "Preparing to create project", 000 , "subproject", severity="DEBUG")
        # create project
        check_project(missing, env)
        create_project(session, missings[missing])

    for orphan in orphans:
        prettyllog("semaphore", "delete proejct", orphan, "Preparing to delete project", 000 , "subproject", severity="DEBUG")
        # delete project
        # delete project in git
        # delete project in semaphore
        # delete project in awx
        # delete project in vault
        # delete project in dns

    # prettyllog("semaphore", "main", "main", "ok", 0 , "subproject", severity="INFO")

    projects = get_project(session)
    for project in projects:
        prettyllog("semaphore", "projects", project, "ok", 0 , "project", severity="INFO")
        projectname = projects[project]['name']
        state[projectname] = {}
        state[projectname]['project'] = projects[project]
        state[projectname]['inventory'] = {}
        inventory = get_inventory(session, projects[project]['id'])
        for item in inventory:
            itemname = inventory[item]['name']
            state[projectname]['inventory'][itemname] = {}
            state[projectname]['inventory'][itemname]['item'] = inventory[item]
            prettyllog("semaphore", "main", item, "ok", 0 , "item", severity="INFO")



        




    return 0




# keep ign8 and con5entrate on your semaphores

from . import serve
import argparse

def main():
    parser = argparse.ArgumentParser(description="Keep ign8 and con5entrate on your semaphores", usage="ign8_semaphore <action> \n\n \
\
version : 0.0.2 (semaphore)\n\
actions:\n\
serve      keep ign8 and serve semaphore\n\
init       keep ign8 and init semaphore systemd service\n\
start      keep ign8 and start semaphore systemd service\n\
stop       keep ign8 and stop semaphore systemd service\n\
restart    keep ign8 and restart semaphore systemd service\n\
setup      keep ign8 and setup semaphore\n\
test       keep ign8 and test semaphore\n\
audit      keep ign8 and audit semaphore\n\
\n\
\
2023 Knowit Miracle\
")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False

    if args.action[0] == "serve":
        serve.main()
        return 0
    




import os
import requests
import datetime
from .common import prettyllog
from .netbox.netbox import check_netbox


def main():
    prettyllog("main", "check", "main", "all", "200", "Success")
    return True

def getenvironmentvalue(myenv):
    myenv = os.getenv(myenv, "false")
    return myenv

def getenvironment():
    environment = {}
    environment["IGN8_NETBOX_URL"] = getenvironmentvalue("IGN8_NETBOX_URL")
    environment["IGN8_NETBOX_TOKEN"] = getenvironmentvalue("IGN8_NETBOX_TOKEN")
    environment["IGN8_NETBOX_VERIFY_SSL"] = getenvironmentvalue("IGN8_NETBOX_VERIFY_SSL")
    if check_netbox():
        return environment
    else:
        return False


def setupign8():
    prettyllog("setup", "check", "access", "all", "200", "Success")
    myenv = getenvironment()
    if myenv:
        prettyllog("setup", "check", "access", "all", "200", "Success")
        return True
    else:
        prettyllog("setup", "check", "setupign8", "all", "500", "Failed", "ERROR")
        return False    




from .awx_common import awx_get_id
from .awx_common import getawxdata

from ..common import prettyllog

import requests
import os
import json



VERIFY_SSL = os.getenv("VERIFY_SSL", "false")
if VERIFY_SSL == "false" or VERIFY_SSL == "False" or VERIFY_SSL == "FALSE" or VERIFY_SSL == "no" or VERIFY_SSL == "NO" or VERIFY_SSL == "No":
  VERIFY_SSL = False
else:
  VERIFY_SSL = True




def awx_create_executionenvironment(ee , organization, mytoken, r):
  try:
    eeid = (awx_get_id("execution_environments", ee['name'], r))
  except:
    print("Unexcpeted credential error")
  orgid = (awx_get_id("organizations", organization, r))
  headers = {"User-agent": "python-awx-client", "Content-Type": "application/json","Authorization": "Bearer {}".format(mytoken)}

  data = {
      "name": ee['name'],
      "image": ee['image'],
      "description": ee['description'],
      "pull": ee['pull'],
      "registry_credential": ee['registry_credential'],
      "organization": orgid
    }
  if ( eeid == ""):
    url = os.getenv("TOWER_HOST") + "/api/v2/execution_environments/"
    resp = requests.post(url,headers=headers, json=data, verify=VERIFY_SSL)
    response = json.loads(resp.content)
    try:
      credid=response['id']
      prettyllog("manage", "execution_environments", ee['name'], organization, resp.status_code, credid, "INFO")
    except:
      prettyllog("manage", "execution_environments", ee['name'], organization, resp.status_code, response, "ERROR")
  else:
    url = os.getenv("TOWER_HOST") + "/api/v2/execution_environments/%s/" % eeid
    resp = requests.put(url,headers=headers, json=data, verify=VERIFY_SSL)
    response = json.loads(resp.content)
    try:
      eeid=response['id']
      prettyllog("manage", "execution_environments", ee['name'], organization, resp.status_code, eeid, "INFO")
    except:
      prettyllog("manage", "execution_environments", ee['name'], organization, resp.status_code, response, "ERROR")
  getawxdata("execution_environments", mytoken, r)





"""
Written by Michael Rice
Github: https://github.com/michaelrice
Website: https://michaelrice.github.io/
Blog: http://www.errr-online.com/
This code has been released under the terms of the Apache-2.0 license
http://opensource.org/licenses/Apache-2.0

Example to show the minimal steps required to create a cluster in
vCenter.

Required Privileges
Host.Inventory.CreateCluster

Since
VI API 2.5
"""
from pyVmomi import vim


def create_cluster(**kwargs):
    """
    Method to create a Cluster in vCenter

    :param kwargs:
    :return: Cluster MORef
    """
    cluster_name = kwargs.get("name")
    cluster_spec = kwargs.get("cluster_spec")
    datacenter = kwargs.get("datacenter")

    if cluster_name is None:
        raise ValueError("Missing value for name.")
    if datacenter is None:
        raise ValueError("Missing value for datacenter.")
    if cluster_spec is None:
        cluster_spec = vim.cluster.ConfigSpecEx()

    host_folder = datacenter.hostFolder
    cluster = host_folder.CreateClusterEx(name=cluster_name, spec=cluster_spec)
    return cluster



import requests
import time
import os
import urllib3
import redis
from PIL import Image
import subprocess
import hashlib
import hvac


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
r=redis.Redis()


def init_redis():
  r = redis.Redis()
  r.set('foo', 'bar')
  value = r.get('foo')
  if value == b'bar':
    print("Redis is working")
    return r
  else:
    print("Redis is not working")
    exit(1)

pitvurl = os.getenv("pitv_URL", "https://pitvapi.openknowit.com")  

def status():
  print(pitvurl)

def service():
  print("service")



def calculate_md5(filename, block_size=65536):
    if os.path.isfile(filename):
      md5_hash = hashlib.md5()
      try:
        with open(filename, "rb") as file:
          for block in iter(lambda: file.read(block_size), b""):
            md5_hash.update(block)
        return md5_hash.hexdigest()
      except:
        return None
    else:
      return None


def hashit(file):
   #read file in binary mode
    with open(file, 'rb') as f:
        data = f.read()
        #return md5 hash
        data = data.encode("utf-8")
        return hashlib.md5(data).hexdigest()

def check_if_file_is_picture(file):
  print("check if file is a picture")
  file = file.lower()
  if file.endswith(".jpg") or file.endswith(".jpeg") or file.endswith(".png") or file.endswith(".gif") or file.endswith(".bmp") or file.endswith(".tiff") or file.endswith(".tif") or file.endswith(".webp") or file.endswith("cr2"):
    
    return True
  else:
    return False

def get_image_metadata(image_path):
    try:
        with Image.open(image_path) as img:
            metadata = img.info
            if len(metadata) > 0:
              return metadata
            else:
              return None
    except Exception as e:
        print("Error:", e)
        return None

def list_files_recursive(directory):
    files = []
    for root, dirs, files in os.walk(directory):
        for filename in files:
            file_path = os.path.join(root, filename)
            try:
                with open(file_path, "rb"):
                    files.append(file_path)
            except PermissionError:
                print("Permission denied for file:", file_path)
    return files

def locate_files(keyword=""):
    try:
        # Run the locate command and capture the output
        command = ["locate", keyword]
        result = subprocess.run(command, stdout=subprocess.PIPE, text=True, check=True)
        output = result.stdout
        file_list = output.splitlines()
        return file_list
    except subprocess.CalledProcessError as e:
        print("Error:", e)
        return []

def evacuate():
  redis = init_redis()
  #get all files on the system
  files = locate_files()
  total = len(files)
  count = 0
  for file in files:
    count = count + 1
    if redis.exists(file):
      # get file size
      try:
        filesize = os.path.getsize(file)
      except:
        filesize = 0
        
      status = redis.get(file).decode("utf-8")
      # print a status wihout newline
      print("file " + str(count) + " of " + str(total) + " status: " + status, end="\r")
      md5 = calculate_md5(file)
      print(md5)
      # scp file remotehost:/files/
      if md5 is not None: 
        keys = "MD5:" + md5
        vault.secrets.kv.v2.existing_version(keys)
        if vault.secrets.kv.v2.get_secret_version(keys) is None:
          vault.secrets.kv.v2.create_or_update_secret(keys, md5=md5, file=file)
        
        redis.set(keys, filesize)

      if status == "1":
        metadata = get_image_metadata(file)
        if metadata is not None:
           key = "metadata:" + file
           redis.set(key, str(metadata))
    else:
      print("file " + str(count) + " of " + str(total) + " status: unknown", end="\r")
      if check_if_file_is_picture(file):
        print(file)
        key = "Picture:" + file
        redis.set(file, "1")
      else:
        redis.set(file, "999")
      
    





#      if status == 0 or status == 
#        print("file " + str(count) + " of " + str(total) ) #no newline
#        if check_if_file_is_picture(file):
#          print(file)
#          key = "Picture:" + file
#         redis.set(file, "1")
#       else:
#         redis.set(file, "999")
#      if status == "1":
#        metadata = get_image_metadata(file)
#        print(metadata)
#      else:
#        redis.set(file, "0")
#  print("evacuate")
  #get all files on the system

  



"""
This module implements simple helper functions for managing service instance objects
"""
__author__ = "VMware, Inc."

import atexit
from pyVim.connect import SmartConnect, Disconnect


def connect(args):
    """
    Determine the most preferred API version supported by the specified server,
    then connect to the specified server using that API version, login and return
    the service instance object.
    """

    service_instance = None

    # form a connection...
    try:
        if args.disable_ssl_verification:
            service_instance = SmartConnect(host=args.host,
                                            user=args.user,
                                            pwd=args.password,
                                            port=args.port,
                                            disableSslCertValidation=True)
        else:
            service_instance = SmartConnect(host=args.host,
                                            user=args.user,
                                            pwd=args.password,
                                            port=args.port,
                                            disableSslCertValidation=True)

        # doing this means you don't need to remember to disconnect your script/objects
    except IOError as io_error:
        print(io_error)

    if not service_instance:
        raise SystemExit("Unable to connect to host with supplied credentials.")

    return service_instance




from django.urls import path, include
from .views import selinux_list, selinux_event_list , UploadSelinuxDataView, UploadSElinuxEventView
from rest_framework.routers import DefaultRouter



urlpatterns = [
    path('', selinux_list, name='selinux_list'),
    path('selinux_list/', selinux_list, name='selinux_list'),
    path('selinux_event_list/', selinux_event_list, name='selinux_event_list'),
    path('selinux_event_list/<int:pk>/', selinux_event_list, name='selinux_event_list'),
    path('upload_selinux_data/', UploadSelinuxDataView.as_view(), name='upload_selinux_data'),
    path('upload_selinux_event/', UploadSElinuxEventView.as_view(), name='upload_selinux_event'),
]



from . import netbox
import argparse

def main():
    parser = argparse.ArgumentParser(description="Keep ign8 and automate netbox", usage="ign8_netbox <action> \n\n \
               \
                version : 0.0.2 (netbox)\n\
                actions:\n\
                netboxdata                dump netbox data in json \n\
                inventory_upload          upload ansible inventory to netbox \n\
                ssh_config                dump ssh_config\n\
                ansible_inventory         dump ansible inventory\n\
                devices_types             list device types\n\
                manufacturers             list manufacturers\n\
                tenants                   list tenants\n\
                roles                     list roles\n\
                sites                     list sites\n\
                ipranges                  list ip ranges\n\
                ips                       list ips\n\
                create_cluster            create cluster\n\
                create_tenant_group       create tenant group\n\
                create_tenant             create tenant\n\
                create_manufacturer       create manufacturer\n\
                create_site               create site\n\
                create_role               create role\n\
                create_ip                 create ip address\n\
                create_iprange            create ip range address\n\
                create_device_type        create device type\n\
                add_vm                    add virtual server to netbox\n\
                add_device                add device to netbox\n\
                refresh                   refresh core netbox content\n\
                get_config                get config\n\
                serve                     \n\
               \
               2023 Knowit Miracle\
               ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False

    if args.action[0] == "serve":
        print("serve mode manintaon netbox data")
        netbox.serve()
        return 0

    if args.action[0] == "inventory_upload":
        print("inventory_upload")
        netbox.inventory_upload()
        return 0
    
    if args.action[0] == "help":
        print(parser.print_help())
        return 0
    if args.action[0] == "version":
        print("0.0.2")
        return 0
    if args.action[0] == "test":
        print("test")
        return 0
    if args.action[0] == "setup":
        netbox.setup()
        return 0

    if args.action[0] == "devices_types":
        print(netbox.get_device_types())
        return 0
    
    if args.action[0] == "manufacturers":
        print(netbox.get_manufacturers())
        return 0
    
    if args.action[0] == "sites":
        print(netbox.get_sites())
        return 0
    

    if args.action[0] == "refresh":
        netbox.refresh()
        return 0
    
    if args.action[0] == "create_iprange":
        netbox.create_iprange(args)
        return 0

    if args.action[0] == "create_ip":
        netbox.create_ip4(args)
        return 0
    
    if args.action[0] == "create_cluster":
        netbox.create_cluster(args)
        return 0
    if args.action[0] == "create_tenant_group":
        netbox.create_tenant_group(args)
        return 0
    
    if args.action[0] == "create_tenant":
        netbox.create_tenant(args)
        return 0
    
    if args.action[0] == "create_device_type":
        netbox.create_device_type(args)
        return 0
    
    if args.action[0] == "create_role":
        netbox.create_role()
        return 0
    
    if args.action[0] == "create_site":
        netbox.create_site(args)
        return 0
    
    if args.action[0] == "create_manufacturer":
        netbox.create_manufacturer(args)
        return 0
    
    if args.action[0] == "ansible_inventory":
        netbox.ansible_inventory(args)
        return 0
    
    if args.action[0] == "ssh_config":
        netbox.sshconfig(args)
        return 0
    
    if args.action[0] == "netboxdata":
        netbox.netboxdata(args)
        return 0
    
    if args.action[0] == "vizualize":
        netbox.vizulize(args)
        return     
    
    if args.action[0] == "sites":
        print(netbox.get_sites())
        return 0
    
    if args.action[0] == "tenants":
        print(netbox.get_tenants())
        return 0
    
    if args.action[0] == "roles":
        print(netbox.get_roles())
        return 0
    
    if args.action[0] == "add_vm":
        return netbox.add_vm()    
    
    if args.action[0] == "add_device":
        print("add_device")
        return netbox.add_device()    

    if args.action[0] == "get_config":
        print(netbox.get_config())
        return 0
    









    
    return 0
    

    

    










import requests
import json
import os
import base64
import xml.etree.ElementTree as ET
import platform
import yaml
from ..common import prettyllog

# we need to suppress ssl warnings
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

#netbox_url = os.environ.get('NETBOX_URL')
#netbox_token = os.environ.get('NETBOX_TOKEN')

NETBOX_URL = os.getenv("IGN8_NETBOX_URL")
NETBOX_TOKEN = os.getenv("IGN8_NETBOX_TOKEN")

ssh_config_template = """
Host {hostname}
    HostName {full_hostname}
    User root
    IdentityFile ~/.ssh/disposeablekey
    IdentityFile ~/.ssh/disposeablekey.signed
    Port 22
    {proxy_jump}
"""


def check_netbox():
    prettyllog("manage", "netbox", "check", "all", "000", "Checking NetBox")
    try:
        response = requests.get(NETBOX_URL, verify=False)
        if response.status_code == 200:
            prettyllog("check", "netbox", "access", "all", "200", "Success", "INFO")
            return True
        else:
            prettyllog("check", "netbox", "access", "all", "500", "Failed", "ERROR")
            return False
    except:
        prettyllog("manage", "netbox", "check", "all", "500", "Failed", "ERROR")
        return False
    

def vizulize(args):
    cluseters = get_clusters()
    vms = get_virtual_machines()
    print("digraph G {")
    for cluster in cluseters:
        print(f"    {cluster['name']}")
    for vm in vms:
        print(f"    {vm['name']}")
        if vm.get("cluster"):
            print(f"    {vm['name']} -> {vm['cluster']['name']}")
    print("}")

def get_ip4_id(ip4_address):
    ip4s = get_ip4s()
    try:
        return ip4s[ip4_address]
    except:
        return None
    
def get_ip6_id(ip6_address):
    ip6s = get_ip6s()
    try:
        return ip6s[ip6_address]
    except:
        return None
    

def get_site_id(site_name):
    sites = get_sites()
    try:
        return sites[site_name]
    except:
        return None

def get_device_id(device_name):
    prettyllog("manage", "netbox", "device", "new", "000", "Getting device id")
    devices = get_devices()
    try:    
        prettyllog("manage", "netbox", "device", "new", "000", "Device id is %s" % devices[device_name])
        return devices[device_name]
    except:
        prettyllog("manage", "netbox", "device", "new", "000", "Device id is None")
        return None

def get_role_id(role_name):
    roles = get_roles()
    try: 
        return roles[role_name]
    except:
        return None


def get_type_id(type_name):
    types = get_types()
    try:
        return types[type_name]
    except:
        return None

def get_platform_id(platform_name):
    platforms = get_platforms()
    try:
        return platforms[platform_name]
    except:
        return None

def get_tenant_id(tenant_name):
    tenants = get_tenants()
    try:
        return tenants[tenant_name]
    except:
        return None

def get_cluster_id(cluster_name):
    clusters = get_clusters()
    try: 
        return clusters[cluster_name]
    except:
        return None

def get_manufacturer_id(manufacturer_name):
    manufacturers = get_manufacturers()
    try:
        return manufacturers[manufacturer_name]
    except:
        return None
        
def get_virtual_machine_id(vm_name):
    vms = get_virtual_machines()
    try:
        return vms[vm_name]
    except:
        return None
    
def get_interface_id(interface_name):
    interfaces = get_interfaces()
    try:
        return interfaces[interface_name]
    except:
        return None
    

def create_manufacturer(manufacturer_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    manufacturer_name = os.environ.get('IGN8_MANUFACTURER_NAME')
    if manufacturer_name == None:
        manufacturer_name = "default"

    manufacturer_slug = os.environ.get('IGN8_MANUFACTURER_SLUG')
    if manufacturer_slug == None:
        manufacturer_name = manufacturer_name.lower()
    manufacturer_description = os.environ.get('IGN8_MANUFACTURER_DESCRIPTION')
    if manufacturer_description == None:
        manufacturer_description = ""
    manufacturer_comments = os.environ.get('IGN8_MANUFACTURER_COMMENTS')
    if manufacturer_comments == None:
        manufacturer_comments = ""
    manufacturer_custom_fields = os.environ.get('IGN8_MANUFACTURER_CUSTOM_FIELDS')
    if manufacturer_custom_fields == None:
        manufacturer_custom_fields = {}

    data = {
        "name": manufacturer_name,
        "slug": manufacturer_name,
        "description": "",
        "comments": "",
        "custom_fields": {}
    }
    manufacturer_id = get_manufacturer_id(manufacturer_name)
    if manufacturer_id != None:
        return True

    url = fix_url("/dcim/manufacturers/")
    response = requests.post(url, headers=headers, json=data)
    print(response)
    print(response.json())
    print(response.status_code)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False
    


def create_device_type(device_type_name):
    prettyllog("manage", "netbox", "device", "new", "000", "Creating device type")
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    prettyllog("manage", "netbox", "device", "new", "000", "Getting device type name")
    device_type_name = os.environ.get('IGN8_DEVICE_TYPE_NAME')
    if device_type_name == None:
        device_type_name = "default"

    device_type_model = os.environ.get('IGN8_DEVICE_TYPE_MODEL')
    if device_type_model == None:
        device_type_model = "default"
    device_type_slug = os.environ.get('IGN8_DEVICE_TYPE_SLUG')
    if device_type_slug == None:
        device_type_slug = device_type_name.lower()
    device_type_description = os.environ.get('IGN8_DEVICE_TYPE_DESCRIPTION')
    if device_type_description == None:
        device_type_description = ""
    device_type_comments = os.environ.get('IGN8_DEVICE_TYPE_COMMENTS')
    if device_type_comments == None:
        device_type_comments = ""
    device_type_custom_fields = os.environ.get('IGN8_DEVICE_TYPE_CUSTOM_FIELDS')
    if device_type_custom_fields == None:

        device_type_custom_fields = {}
    # Optional fields
    device_type_part_number = os.environ.get('IGN8_DEVICE_TYPE_PART_NUMBER')
    if device_type_part_number == None:
        device_type_part_number = ""
    device_type_is_full_depth = os.environ.get('IGN8_DEVICE_TYPE_IS_FULL_DEPTH')
    if device_type_is_full_depth == None:
        device_type_is_full_depth = True
    device_type_is_console_server = os.environ.get('IGN8_DEVICE_TYPE_IS_CONSOLE_SERVER')
    if device_type_is_console_server == None:
        device_type_is_console_server = False
    device_type_is_pdu = os.environ.get('IGN8_DEVICE_TYPE_IS_PDU')
    if device_type_is_pdu == None:
        device_type_is_pdu = False
    device_type_is_network_device = os.environ.get('IGN8_DEVICE_TYPE_IS_NETWORK_DEVICE')
    if device_type_is_network_device == None:
        device_type_is_network_device = False
    device_type_subdevice_role = os.environ.get('IGN8_DEVICE_TYPE_SUBDEVICE_ROLE')
    if device_type_subdevice_role == None:
        device_type_subdevice_role = "" 
    device_type_interface_ordering = os.environ.get('IGN8_DEVICE_TYPE_INTERFACE_ORDERING')
    if device_type_interface_ordering == None:
        device_type_interface_ordering = None
    device_type_tags = os.environ.get('IGN8_DEVICE_TYPE_TAGS')
    if device_type_tags == None:
        device_type_tags = []
    device_type_manufacturer = os.environ.get('IGN8_DEVICE_TYPE_MANUFACTURER')
    if device_type_manufacturer == None:
        device_type_manufacturer = "default"
    device_type_manufacturer_id = get_manufacturer_id(device_type_manufacturer)
    if device_type_manufacturer_id == None:
        create_manufacturer(device_type_manufacturer)
        device_type_manufacturer_id = get_manufacturer_id(device_type_manufacturer)
    device_type_height = os.environ.get('IGN8_DEVICE_TYPE_HEIGHT')
    if device_type_height == None:
        device_type_height = 1

    data = {
        "name": device_type_name,
        "model": device_type_model,
        "manufacturer": device_type_manufacturer_id,
        "slug": device_type_slug,
        "u_height": device_type_height,
        "is_full_depth": device_type_is_full_depth,
        "is_console_server": device_type_is_console_server,
        "is_pdu": device_type_is_pdu,
        "is_network_device": device_type_is_network_device,
        "subdevice_role": device_type_subdevice_role,
        "interface_ordering": device_type_interface_ordering,
        "comments": device_type_comments,
        "tags": device_type_tags,
        "custom_fields": device_type_custom_fields
    }
    device_type_id = get_type_id(device_type_name)
    if device_type_id != None:
        return True
    url = fix_url("/dcim/device-types/")
    response = requests.post(url, headers=headers, json=data)
    print(response)
    print(response.json())
    print(response.status_code)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False
    
def create_device():
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }

    # Mandatory fields
    device_name = os.environ.get('IGN8_DEVICE_NAME')
    if device_name == None:
        device_name = "default"
    device_type = os.environ.get('IGN8_DEVICE_TYPE')
    if device_type == None:
        device_type = "default"
    device_role = os.environ.get('IGN8_DEVICE_ROLE')
    if device_role == None:
        device_role = "default"
    device_site = os.environ.get('IGN8_DEVICE_SITE')
    if device_site == None:
        device_site = "default"
    device_status = os.environ.get('IGN8_DEVICE_STATUS')
    if device_status == None:
        device_status = "active"
    device_comments = os.environ.get('IGN8_DEVICE_COMMENTS')
    if device_comments == None:
        device_comments = ""
    devicetype_id = get_type_id(device_type)
    if devicetype_id == None:
        create_device_type(device_type)
        devicetype_id = get_type_id(device_type)
    device_role_id = get_role_id(device_role)
    if device_role_id == None:
        create_role(device_role)
        device_role_id = get_role_id(device_role)
    device_site_id = get_site_id(device_site)
    if device_site_id == None:
        create_site(device_site)
        device_site_id = get_site_id(device_site)



    data = {
        "name": device_name,
        "device_type": 1,
        "device_role": 1,
        "site": 1,
        "status": "active",  
        "comments": "This is an example device in NetBox."
    }

def create_role():
    prettyllog("manage", "netbox", "device", "new", "000", "Creating role")
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role name")
    role_name = os.environ.get('IGN8_ROLE_NAME')
    if role_name == None:
        role_name = "default"
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role name is %s" % role_name)
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role slug")
    role_slug = os.environ.get('IGN8_ROLE_SLUG')
    if role_slug == None:
        role_slug = role_name.lower()
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role slug is %s" % role_slug)
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role description")
    role_description = os.environ.get('IGN8_ROLE_DESCRIPTION')
    if role_description == None:
        role_description = ""
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role description is %s" % role_description)
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role comments")

    role_comments = os.environ.get('IGN8_ROLE_COMMENTS')
    if role_comments == None:
        role_comments = ""
    prettyllog("manage", "netbox", "device", "new", "000", "Getting role comments is %s" % role_comments)
    prettyllog("manage", "netbox", "device", "new", "000", "assemble data")
    data = {
        "name": role_name,
        "slug": role_slug,
        "description": role_description,
        "comments": role_comments
    }
    role_id = get_role_id(role_name)
    if role_id != None:
        prettyllog("manage", "netbox", "device", "new", "000", "Role already exists")
        return True
    url = fix_url("/dcim/device-roles/")
    response = requests.post(url, headers=headers, json=data)
    print(response)
    print(response.json())
    print(response.status_code)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False

def create_platform(platform_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    platform_name = os.environ.get('IGN8_PLATFORM_NAME')
    if platform_name == None:
        platform_name = "default"
    platform_slug = os.environ.get('IGN8_PLATFORM_SLUG')
    if platform_slug == None:
        platform_slug = platform_name.lower()
    platform_description = os.environ.get('IGN8_PLATFORM_DESCRIPTION')
    if platform_description == None:
        platform_description = ""
    platform_comments = os.environ.get('IGN8_PLATFORM_COMMENTS')
    if platform_comments == None:
        platform_comments = ""



def create_site(site_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    site_name = os.environ.get('IGN8_SITE_NAME')
    if site_name == None:
        site_name = "default"
    site_slug = os.environ.get('IGN8_SITE_SLUG')
    if site_slug == None:
        site_slug = site_name.lower()
    site_description = os.environ.get('IGN8_SITE_DESCRIPTION')
    if site_description == None:
        site_description = ""
    site_comments = os.environ.get('IGN8_SITE_COMMENTS')
    if site_comments == None:
        site_comments = ""
    site_custom_fields = os.environ.get('IGN8_SITE_CUSTOM_FIELDS')
    if site_custom_fields == None:
        site_custom_fields = {}

    data = {
        "name": site_name,
        "slug": site_slug,
        "description": site_description,
        "comments": site_comments,
        "custom_fields": site_custom_fields
    }   
    site_id = get_site_id(site_name)
    if site_id != None:
        return True
    url = fix_url("/dcim/sites/")
    response = requests.post(url, headers=headers, json=data)
    print(response)
    print(response.json())
    print(response.status_code)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False



def create_manufacturer(manufacturer_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    manufacturer_name = os.environ.get('IGN8_MANUFACTURER_NAME')
    if manufacturer_name == None:
        manufacturer_name = "default"

    manufacturer_slug = os.environ.get('IGN8_MANUFACTURER_SLUG')
    if manufacturer_slug == None:
        manufacturer_name = manufacturer_name.lower()
    manufacturer_description = os.environ.get('IGN8_MANUFACTURER_DESCRIPTION')
    if manufacturer_description == None:
        manufacturer_description = ""
    manufacturer_comments = os.environ.get('IGN8_MANUFACTURER_COMMENTS')
    if manufacturer_comments == None:
        manufacturer_comments = ""
    manufacturer_custom_fields = os.environ.get('IGN8_MANUFACTURER_CUSTOM_FIELDS')
    if manufacturer_custom_fields == None:
        manufacturer_custom_fields = {}

    data = {
        "name": manufacturer_name,
        "slug": manufacturer_name,
        "description": "",
        "comments": "",
        "custom_fields": {}
    }
    manufacturer_id = get_manufacturer_id(manufacturer_name)
    if manufacturer_id != None:
        return True
    
    url = fix_url("/dcim/manufacturers/")
    response = requests.post(url, headers=headers, json=data)
    print(response)
    print(response.json())
    print(response.status_code)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False



def create_type(type_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    manufacturer = os.environ.get("IGN8_NETBOX_MANUFACTURER")
    if manufacturer == None:
        manufacturer == "noname"
    manufacturer_id = get_manufacturer_id(manufacturer)
    if manufacturer_id == None:
        create_manufacturer(manufacturer)
        manufacturer_id = get_manufacturer_id(manufacturer)

    data = {
        "name": type_name, 
        "model": type_name,
        "manufacturer": manufacturer_id,
        "slug": type_name,
        "u_height": 1,
        "is_full_depth": True,
        "is_console_server": False,
        "is_pdu": False,
        "is_network_device": False,
        "subdevice_role": None,
        "interface_ordering": None,
        "comments": "",
        "tags": [],
        "custom_fields": {}
    }

    url = fix_url("/dcim/device-types/")
    response = requests.post(url, headers=headers, json=data)
    if response.status_code == 200:
        return True
    else:
        return False
    
def create_platform(platform_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    platform_name = os.environ.get('IGN8_PLATFORM_NAME')
    if platform_name == None:
        platform_name = "default"
    platform_slug = os.environ.get('IGN8_PLATFORM_SLUG')
    if platform_slug == None:
        platform_slug = platform_name.lower()
    platform_description = os.environ.get('IGN8_PLATFORM_DESCRIPTION')
    if platform_description == None:
        platform_description = ""
    platform_comments = os.environ.get('IGN8_PLATFORM_COMMENTS')
    if platform_comments == None:
        platform_comments = ""
    platform_napalm_driver = os.environ.get('IGN8_PLATFORM_NAPALM_DRIVER')
    if platform_napalm_driver == None:
        platform_napalm_driver = ""
    platform_napalm_args = os.environ.get('IGN8_PLATFORM_NAPALM_ARGS')
    if platform_napalm_args == None:
        platform_napalm_args = ""
    data = {
        "name": platform_name,
        "slug": platform_slug,
        "napalm_driver": platform_napalm_driver,
        "napalm_args": platform_napalm_args,
        "description": platform_description,
        "comments": platform_comments
    }
    platform_id = get_platform_id(platform_name)
    if platform_id != None:
        return True
    url = fix_url("/dcim/platforms/")
    response = requests.post(url, headers=headers, json=data)
    print(response)
    print(response.json())
    print(response.status_code)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False
    
def create_virtual_machine(vm_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    vm_name = os.environ.get('IGN8_VM_NAME')
    if vm_name == None:
        vm_name = "default"

    vm_cluster = os.environ.get('IGN8_VM_CLUSTER')
    if vm_cluster == None:
        vm_cluster = "default"
    vm_tenant = os.environ.get('IGN8_VM_TENANT')
    if vm_tenant == None:
        vm_tenant = "default"
    vm_role = os.environ.get('IGN8_VM_ROLE')
    if vm_role == None:
        vm_role = "default"
    vm_status = os.environ.get('IGN8_VM_STATUS')
    if vm_status == None:
        vm_status = "active"
    vm_comments = os.environ.get('IGN8_VM_COMMENTS')
    if vm_comments == None:
        vm_comments = ""
    vm_custom_fields = os.environ.get('IGN8_VM_CUSTOM_FIELDS')
    if vm_custom_fields == None:
        vm_custom_fields = {}
    vm_type = os.environ.get('IGN8_VM_TYPE')
    if vm_type == None:
        vm_type = "default"
    vm_type_id = get_type_id(vm_type)
    if vm_type_id == None:
        create_type(vm_type)
        vm_type_id = get_type_id(vm_type)
    vm_tenant_id = get_tenant_id(vm_tenant)
    if vm_tenant_id == None:
        create_tenant(vm_tenant)
        vm_tenant_id = get_tenant_id(vm_tenant)
    vm_role_id = get_role_id(vm_role)
    if vm_role_id == None:
        create_role(vm_role)
        vm_role_id = get_role_id(vm_role)
    vm_cluster_id = get_cluster_id(vm_cluster)
    if vm_cluster_id == None:
        create_cluster(vm_cluster)
        vm_cluster_id = get_cluster_id(vm_cluster)
    data = {
        "name": vm_name,
        "cluster": vm_cluster_id,
        "tenant": vm_tenant_id,
        "role": vm_role_id,
        "status": vm_status,
        "comments": vm_comments,
        "custom_fields": vm_custom_fields,
        "type": vm_type_id
    }
    vm_id = get_virtual_machine_id(vm_name)
    if vm_id != None:
        return True
    url = fix_url("/virtualization/virtual-machines/")
    response = requests.post(url, headers=headers, json=data)
    if response.status_code == 200:
        return True
    else:
        return False
    
def create_iprange(iprange_description = "default"):
    prettyllog("manage", "netbox", "iprange", "new", "000", "Creating iprange")
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange start address")
    ip_start_address = os.environ.get('IGN8_IP_START_ADDRESS')
    if ip_start_address == None:
        ip_start_address = ""
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange end address")
    ip_end_address = os.environ.get('IGN8_IP_END_ADDRESS')
    if ip_end_address == None:
        ip_end_address = ""
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange status")
    ip_status = os.environ.get('IGN8_IP_STATUS')
    if ip_status == None:
        ip_status = "active"
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange vrf")

    ip_vrf = os.environ.get('IGN8_IP_VRF')
    if ip_vrf == None:
        ip_vrf = "Global"
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange role")
    ip_role = os.environ.get('IGN8_IP_ROLE')
    if ip_role == None:
        ip_role = "loopback"

    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange tenant")
    ip_tenant = os.environ.get('IGN8_IP_TENANT')
    if ip_tenant == None:
        ip_tenant = "default"
    ip_tenant_id = get_tenant_id(ip_tenant)
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange tenana id")
    if ip_tenant_id == None:
        create_tenant(ip_tenant)
        ip_tenant_id = get_tenant_id(ip_tenant)
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange description")
    iprange_description = os.environ.get('IGN8_IPRANGE_DESCRIPTION')
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange description is %s" % iprange_description)
    prettyllog("manage", "netbox", "iprange", "new", "000", "Getting iprange iprange id")
    iprange_description_id = get_iprange_id(iprange_description)
    if iprange_description_id != None:
        prettyllog("manage", "netbox", "iprange", "new", "000", "Ip range already exists")
        return True


#            "id": 2,
#            "url": "http://netbox.openknowit.com/api/ipam/ip-ranges/2/",
#            "display": "192.168.86.0-255/24",
#            "family": {
#                "value": 4,
#                "label": "IPv4"
#            },
#            "start_address": "192.168.86.0/24",
#           "end_address": "192.168.86.255/24",
#            "size": 256,
#            "vrf": null,
#            "tenant": null,
#            "status": {
#                "value": "active",
#                "label": "Active"
#            },
#            "role": null,
#            "description": "",
#            "comments": "",
#           "tags": [],
#            "custom_fields": {},
#            "created": "2023-09-09T21:00:01.355556Z",
#            "last_updated": "2023-09-09T21:00:01.355576Z",
#            "mark_utilized": false

    data  = {
        "start_address": ip_start_address,
        "end_address": ip_end_address,
        "status": ip_status,
        "vrf": ip_vrf,
        "role": ip_role,
        "tenant": ip_tenant_id,
        "description": iprange_description
    }

    url = fix_url("/ipam/ip-ranges/")
    response = requests.post(url, headers=headers, json=data)
    print(response.content)
    if response.status_code == 200:
        return True
    else:
        return False
    




    
def create_ip4(ip4_address = "default"):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }

    # Mandatory fields
    ip_address = os.environ.get('IGN8_IP_ADDRESS')
    if ip_address == None:
        ip_address = "127.0.0.1"
    ip4_status = os.environ.get('IGN8_IP_STATUS')
    if ip_status == None:  
        ip_status = "active"
    ip4_role = os.environ.get('IGN8_IP4_ROLE')
    if ip_role == None:
        ip_role = "loopback"
    tenant = os.environ.get('IGN8_NETBOX_TENANT')   
    if ip_tenant == None:
        ip_tenant = "default"
    tenant_id = get_tenant_id(ip_tenant)
    if tenant_id == None:
        create_tenant(tenant)
        tenant_id = get_tenant_id(tenant)





def create_virtual_interface(interface_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    interface_name = os.environ.get('IGN8_VINTERFACE_NAME')
    if interface_name == None:
        interface_name = "default"
    interface_type = os.environ.get('IGN8_INTERFACE_TYPE')
    if interface_type == None:
        interface_type = "virtual"
    interface_mac_address = os.environ.get('IGN8_INTERFACE_MAC_ADDRESS')
    if interface_mac_address == None:
        interface_mac_address = ""
    interface_mtu = os.environ.get('IGN8_INTERFACE_MTU')
    if interface_mtu == None:
        interface_mtu = ""
    interface_description = os.environ.get('IGN8_INTERFACE_DESCRIPTION')
    if interface_description == None:
        interface_description = ""
    interface_mode = os.environ.get('IGN8_INTERFACE_MODE')

    

def create_cluster(cluster_name):
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    vmtype = os.environ.get('IGN8_VM_TYPE')
    if vmtype == None:
        vmtype = "default"
    typeid = get_type_id(vmtype)

    data = {
        "name": cluster_name,
        "type": typeid
    }

    url = fix_url("/virtualization/clusters/")
    response = requests.post(url, headers=headers, json=data)
    if response.status_code == 200:
        return True
    else:
        return False
    
def create_tenant(tenant_name):

    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    tenant_name = os.environ.get('IGN8_TENANT_NAME')
    if tenant_name == None:
        tenant_name = "default"
    tenant_slug = os.environ.get('IGN8_TENANT_SLUG')
    if tenant_slug == None:
        tenant_slug = tenant_name.lower()
    tenant_description = os.environ.get('IGN8_TENANT_DESCRIPTION')
    if tenant_description == None:
        tenant_description = ""
    tenant_comments = os.environ.get('IGN8_TENANT_COMMENTS')
    if tenant_comments == None:
        tenant_comments = ""
    tenant_custom_fields = os.environ.get('IGN8_TENANT_CUSTOM_FIELDS')
    if tenant_custom_fields == None:
        tenant_custom_fields = {}
        
    data = {
        "name": tenant_name,
        "slug": tenant_slug,
        "description": tenant_description,
        "comments": tenant_comments,
        "custom_fields": tenant_custom_fields
    }
    tenant_id = get_tenant_id(tenant_name)
    if tenant_id != None:
        return True
    url = fix_url("/tenancy/tenants/")
    response = requests.post(url, headers=headers, json=data)
    if response.status_code == 200:
        return True
    else:
        return False
    
    
def get_system_info():
    system_info = platform.uname()
    system_manufacturer = system_info.system
    is_virtual = "QEMU" in system_manufacturer

    return system_manufacturer, is_virtual

def is_kvm_qemu_host():
    # Check if KVM kernel module is loaded
    kvm_module_loaded = os.path.exists('/dev/kvm')

    # Check if QEMU-KVM process is running
    qemu_process_running = False
    with os.popen('pgrep -f qemu') as process:
        qemu_process_running = process.read().strip() != ''

    # Check for presence of QEMU virtualization tools
    qemu_tools_installed = os.path.exists('/usr/bin/qemu-system-x86_64')

    # Check if virt-what tool is available and if it reports "kvm" or "qemu"
    virt_what_installed = os.path.exists('/usr/sbin/virt-what')
    virt_what_result = None
    if virt_what_installed:
        with os.popen('/usr/sbin/virt-what') as process:
            virt_what_result = process.read().strip()
    return kvm_module_loaded or qemu_process_running or qemu_tools_installed or virt_what_result == "kvm" or virt_what_result == "qemu"

def add_device():
    prettyllog("manage", "netbox", "device", "new", "000", "Adding device to netbox")
    prettyllog("manage", "netbox", "device", "new", "000", "Getting system info")
    manufacturer, is_virtual = get_system_info()
    prettyllog("manage", "netbox", "device", "new", "000", "The system is %s and %s " % (manufacturer, str(is_virtual)))
    if not is_virtual:
        prettyllog("manage", "netbox", "device", "new", "000",  "Not a virtual machine, adding to netbox")
        headers = {
            "Authorization": f"Token {NETBOX_TOKEN}",
            "Accept": "application/json"
        }
        # Mandatory fields
        prettyllog("manage", "netbox", "device", "new", "000", "Getting device name")
        device_name = os.environ.get("IGN8_DEVICE_NAME")
        if device_name == None:
            device_name = os.environ.get('HOSTNAME')
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device name is %s" % device_name)

        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device type")    
        device_type = os.environ.get("IGN8_DEVICE_TYPE")
        if device_type == None:
            device_type = "default"
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device type is %s" % device_type)

        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device role")    
        device_role = os.environ.get("IGN8_DEVICE_ROLE")
        if device_role == None:
            device_role = "default"
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device role is %s" % device_role)   

        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device site")
        device_site = os.environ.get("IGN8_DEVICE_SITE")
        if device_site == None:
            device_site = "default"
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device site is %s" % device_site)

        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device status")
        device_status = os.environ.get("IGN8_DEVICE_STATUS")
        if device_status == None:
            device_status = "active"
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device status is %s" % device_status)


        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device comments")    
        device_comments = os.environ.get("IGN8_DEVICE_COMMENTS")
        if device_comments == None:
            device_comments = ""
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device comments is %s" % device_comments)

        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device type id")
        devicetype_id = get_type_id(device_type)
        if devicetype_id == None:
            prettyllog("manage", "netbox", "device", "new", "000",  "Creating device type")
            create_device_type(device_type)
            devicetype_id = get_type_id(device_type)
        prettyllog("manage", "netbox", "device", "new", "000", "Getting device type id is %s" % devicetype_id)

        prettyllog("manage", "netbox", "device", "new", "000", "Getting device role id")
        device_role_id = get_role_id(device_role)
        if device_role_id == None:
            create_role()
            device_role_id = get_role_id(device_role)
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device role id is %s" % device_role_id)

        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device site id")
        device_site_id = get_site_id(device_site)
        if device_site_id == None:
            create_site(device_site)
            device_site_id = get_site_id(device_site)
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device site id is %s" % device_site_id)

        prettyllog("manage", "netbox", "device", "new", "000",  "Assembling device data")
        data = {
            "name": device_name,
            "device_type": devicetype_id,
            "role": device_role_id,
            "device_role": device_role_id,
            "site": device_site_id,
            "status": device_status,
            "comments": device_comments
        }
        prettyllog("manage", "netbox", "device", "new", "000",  "Assembled device data is %s" % data)
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device id")
        device_id = get_device_id(device_name)
        if device_id != None:
            prettyllog("manage", "netbox", "device", "new", "000",  "Device already exists")
            return True
        prettyllog("manage", "netbox", "device", "new", "000",  "Getting device id is %s" % device_id)

        prettyllog("manage", "netbox", "device", "new", "000",  "Preparing to add device")
        url = fix_url("/dcim/devices/")
        response = requests.post(url, headers=headers, json=data)
        prettyllog("manage", "netbox", "device", "new", "000",  "Api call status is %s" % response.status_code)
        print(response.content)
        if response.status_code == 200:
            return True
        else:
            return False
    else:
        return True
    
        
      




    return False

def add_vm():
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    # Mandatory fields
    kvm_qemu_host = is_kvm_qemu_host()

    manufacturer, is_virtual = get_system_info()
    if manufacturer == "QEMU":
        is_virtual = True
    else:
        is_virtual = False
    if not is_virtual:
      servername = os.environ.get("IGN8_SERVER_NAME")
      if servername == None:
        servername = os.environ.get('HOSTNAME')
      


    if is_virtual:    
      vmname = os.environ.get('IGN8_VM_NAME')
      if vmname == None:
        vmname = os.environ.get('HOSTNAME')
      if vmname == None:  
        raise Exception("No VM name provided (IGN8_VM_NAME)")
      vmcluster = os.environ.get('IGN8_VM_CLUSTER')
      if vmcluster == None:
        raise Exception("No VM cluster provided (IGN8_VM_CLUSTER)")
      # Optional fields
      vmsite = os.environ.get('IGN8_VM_SITE')
      if vmsite == None:
        vmsite = "default"
      vmrole = os.environ.get('IGN8_VM_ROLE')
      if vmrole == None:
        vmrole = "default"
      vmtype = os.environ.get('IGN8_VM_TYPE')
      if vmtype == None:
        vmtype = "default"
      vmplatform = os.environ.get('IGN8_VM_PLATFORM')
      if vmplatform == None:
        vmplatform = "default"
      vmtentant = os.environ.get('IGN8_VM_TENANT')
      if vmtentant == None:
        vmtentant = "default"
      vmdevice = os.environ.get('IGN8_VM_DEVICE')
      if vmdevice == None:
        vmdevice = "default"
      vmstatus = os.environ.get('IGN8_VM_STATUS')
      if vmstatus == None:
        vmstatus = "active"
      vmdisk = os.environ.get('IGN8_VM_DISK')
      if vmdisk == None:
        vmdisk = 1
      vmcpus = os.environ.get('IGN8_VM_CPU')
      if vmcpus == None:
        vmcpus = 1
      vmmemory = os.environ.get('IGN8_VM_MEMORY')
      if vmmemory == None:
        vmmemory = 1
      vmip = os.environ.get('IGN8_VM_IP') 
      if vmip == None:
        vmip = ""
      vmip6 = os.environ.get('IGN8_VM_IP6') 
      if vmip6 == None:
        vmip6 = ""
      vmdescription = os.environ.get('IGN8_VM_DESCRIPTION')
      if vmdescription == None:
        vmdescription = ""

      vmcomments = os.environ.get('IGN8_VM_COMMENTS')
      if vmcomments == None:
        vmcomments = ""

      vmlocal_context_data = os.environ.get('IGN8_VM_LOCAL_CONTEXT_DATA')
      if vmlocal_context_data == None:
        vmlocal_context_data = {}
    
      vmtags = os.environ.get('IGN8_VM_TAGS')
      if vmtags == None:
        vmtags = []
      vmcustom_fields = os.environ.get('IGN8_VM_CUSTOM_FIELDS')
      if vmcustom_fields == None:
        vmcustom_fields = {}




    # Get the IDs from Netbox
    
      cluster_id = get_cluster_id(vmcluster)
      site_id = get_site_id(vmsite)
      device_id = get_device_id(vmdevice)
      role_id = get_role_id(vmrole)
      type_id = get_type_id(vmtype)
      platform_id = get_platform_id(vmplatform)
      tenant_id = get_tenant_id(vmtentant)
      ip4_id = get_ip4_id(vmip)
      ip6_id = get_ip6_id(vmip6)


      data = {
            "name": vmname, 
            "status": vmstatus,
            "site": site_id,
            "cluster": cluster_id,
            "device": device_id,
            "role": role_id,
            "tenant": tenant_id,
            "platform": platform_id,
            "primary_ip4": ip4_id,     
            "primary_ip6": ip6_id,  
            "vcpus": vmcpus,
            "memory":    vmmemory,
            "disk": vmdisk,
            "description": vmdescription,
            "comments": vmcomments,
            "local_context_data": vmlocal_context_data,
            "tags": vmtags,
            "custom_fields": vmcustom_fields
        }
      if os.environ.get('NETBOX_API_URL') == None:
        print("No NETBOX_API_URL provided")
        return False
      if os.environ.get('NETBOX_API_TOKEN') == None:
        print("No NETBOX_API_TOKEN provided")
        return False
      nburl=os.environ.get('NETBOX_API_URL')
      if nburl.endswith("/"):
        nburl = nburl[:-1]
      if not nburl.endswith("/api"):
        nburl = nburl + "/api"
      vmid = get_virtual_machine_id(vmname)
      if vmid != None:
        return True
    
      url = nburl + "/virtualization/virtual-machines/"
      response = requests.post(url, headers=headers, json=data)
      if response.status_code == 200:
        return True
      else:
        return False


def fix_url(apiurl):
    nburl=os.environ.get('NETBOX_API_URL')
    if nburl == None:
        print("No NETBOX_API_URL provided")
        return False

    if nburl.endswith("/"):
        nburl = nburl[:-1]
    if not nburl.endswith("/api"):
        nburl = nburl + "/api"
    nburl = nburl + apiurl
    return nburl

def get_iprange_id(iprange_description):
    ipranges =  get_ipranges()
    try:
        return ipranges[iprange_description]
    except:
        return None


def get_ipranges():
    returnipranges = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/ipam/ip-ranges/")
    response = requests.get(url, headers=headers)
    ipranges = response.json()
    for iprange in ipranges["results"]:
        try:
            if returnipranges[iprange["description"]]:
                print("Duplicate iprange name")
        except:
           returnipranges[iprange["description"]] = iprange["id"]
    return returnipranges



def get_interfaces():
    returninterfaces = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/interfaces/")
    response = requests.get(url, headers=headers)
    interfaces = response.json()
    for interface in interfaces["results"]:
        try:
            if returninterfaces[interface["name"]]:
                print("Duplicate interface name")
        except:
           returninterfaces[interface["name"]] = interface["id"]
    return returninterfaces



def get_roles():
    returnroles = {}
    #GET /api/dcim/device-roles/35/
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/device-roles/")
    response = requests.get(url, headers=headers)
    roles = response.json()
    for role in roles["results"]:
        print(role)
        try:
            if returnroles[role["name"]]:
                print("Duplicate role name")
        except:
           returnroles[role["name"]] = role["id"]
    return returnroles



def get_device_types():
    returntypes = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/device-types/")
    response = requests.get(url, headers=headers)
    types = response.json()
    for type in types["results"]:
        print(type)
        try:
            if returntypes[type["display"]]:
                print("Duplicate type name")
        except:
           returntypes[type["display"]] = type["id"]
    return returntypes

def get_tenants():
    returntenants = {}

    # https://netbox.openknowit.com/api/tenancy/tenants/
    # https://netbox.openknowit.com/api/tenancy/tenants/?limit=1000
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/tenancy/tenants/")
    response = requests.get(url, headers=headers)
    tenants = response.json()
    for tenant in tenants["results"]:
        try:
            if returntenants[tenant["name"]]:
                print("Duplicate tenant name")
        except:
           returntenants[tenant["name"]] = tenant["id"]
    return returntenants

def get_roles():
    returnroles = {}
    #GET /api/dcim/device-roles/35/
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/device-roles/")
    response = requests.get(url, headers=headers)
    roles = response.json()
    for role in roles["results"]:
        try:
            if returnroles[role["name"]]:
                print("Duplicate role name")
        except:
           returnroles[role["name"]] = role["id"]
    return returnroles

def get_ip4s():
    returnip4s = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/ipam/ip-addresses/")
    response = requests.get(url, headers=headers)
    ip4s = response.json()
    for ip4 in ip4s["results"]:
        try:
            if returnip4s[ip4["address"]]:
                print("Duplicate ip4 address")
        except:
           returnip4s[ip4["address"]] = ip4["id"]
    return returnip4s

def get_manufacturers():
    returnmanufacturers = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/manufacturers/")
    response = requests.get(url, headers=headers)
    manufacturers = response.json()
    for manufacturer in manufacturers["results"]:
        try:
            if returnmanufacturers[manufacturer["name"]]:
                print("Duplicate manufacturer name")
        except:
           returnmanufacturers[manufacturer["name"]] = manufacturer["id"]
    return returnmanufacturers



def get_ip6s():
    returnip6s = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/ipam/ip-addresses/")
    response = requests.get(url, headers=headers)




def get_types():
    returntypes = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/device-types/")
    response = requests.get(url, headers=headers)
    types = response.json()
    for type in types["results"]:
        try:
            if returntypes[type["name"]]:
                print("Duplicate type name")
        except:
           returntypes[type["display"]] = type["id"]
    return  returntypes

def get_platforms():
    returnplarforms = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/platforms/")
    response = requests.get(url, headers=headers)
    platforms = response.json()
    for platform in platforms["results"]:
        try:
            if returnplarforms[platform["name"]]:
                print("Duplicate platform name")
        except:
           returnplarforms[platform["name"]] = platform["id"]
    return returnplarforms 

def get_devices():
    returndevices = {}

     #/api/dcim/devices/
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/devices/")
    response = requests.get(url, headers=headers)
    devices = response.json()
    for device in devices["results"]:
        try:
            if returndevices[device["name"]]:
                print("Duplicate device name")
        except:
           returndevices[device["name"]] = device["id"]
    return returndevices


def get_sites():
    returnsites = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/dcim/sites/")
    response = requests.get(url, headers=headers)
    sites = response.json()
    for site in sites["results"]:
        try:
            if returnsites[site["name"]]:
                print("Duplicate site name")
        except:
           returnsites[site["name"]] = site["id"]
    return returnsites

def get_clusters():
    returnclusters = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/virtualization/clusters")
    response = requests.get(url, headers=headers)
    clusters = response.json()
    for cluster in clusters["results"]:
        try:
            if returnclusters[cluster["name"]]:
                print("Duplicate cluster name")
        except:
           returnclusters[cluster["name"]] = cluster["id"]
    return returnclusters

def get_virtual_machine(id):
    returnvm = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/virtualization/virtual-machines/" + str(id))
    response = requests.get(url, headers=headers)
    vm = response.json()
    return vm

def get_virtual_machines():
    returnvms = {}
    headers = {
        "Authorization": f"Token {NETBOX_TOKEN}",
        "Accept": "application/json"
    }
    url = fix_url("/virtualization/virtual-machines/")
    response = requests.get(url, headers=headers)
    vms = response.json()
    for vm in vms["results"]:
        try:
            if returnvms[vm["name"]]:
                print("Duplicate vm name")
        except:
           returnvms[vm["name"]] = vm["id"]
    return returnvms


    
def netboxdata(args):
    clusters = get_clusters()
    vms = get_virtual_machines()
    vm_data = []
    for vm in vms:
        vmdata = get_virtual_machine(vms[vm])
        try:
            cluster = vmdata["cluster"]["name"]
        except:
            cluster = None
        vm_entry = {
            "name": vmdata['name'],
            "cluster": cluster,
            "disk_gb": vmdata["disk"],
            "cpu": vmdata["vcpus"],
            "memory_mb": vmdata["memory"],
            "local_context_data": vmdata["local_context_data"]
        }
        vm_data.append(vm_entry)

    data = {
        "virtual_machines": vm_data
    }
    print(json.dumps(data, indent=2))
    return data

def ansible_inventory(args):
    clusters = get_clusters()
    vms = get_virtual_machines()

    vm_data = []
    for vm in vms:
        vm_entry = {
            "name": vm["name"],
            "cluster": vm["cluster"]["name"] if vm.get("cluster") else "N/A",
            "disk_gb": vm["disk"],
            "cpu": vm["vcpus"],
            "memory_mb": vm["memory"],
            "local_context_data": vm["local_context_data"]
        }
        vm_data.append(vm_entry)

    data = {
        "_meta": {
            "hostvars": {}
        },
        "all": {
            "children": ["clusters"]
        },
        "clusters": {
            "hosts": [cluster["name"] for cluster in clusters]
        }
    }

    for vm in vms:
        if vm.get("cluster"):
            cluster_name = vm["cluster"]["name"]
            if cluster_name not in data:
                data[cluster_name] = {"hosts": []}
            data[cluster_name]["hosts"].append(vm["name"])
            data["_meta"]["hostvars"][vm["name"]] = {
                "disk_gb": vm["disk"],
                "cpu": vm["vcpus"],
                "memory_mb": vm["memory"]
            }

    print(yaml.dump(data))


def sshconfig(args):    
    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "Starting ssh config update", "INFO")
    data = netboxdata(args)
    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "Got netbox data", "INFO")
    virtual_machines = data["virtual_machines"]
    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "Got virtual machines", "INFO")
    ssh_config_entries = [generate_ssh_config_entry(vm) for vm in virtual_machines]
    ssh_config_content = "\n".join(ssh_config_entries)
    MYHOME = os.getenv("HOME")
    configdir = os.path.expanduser(MYHOME + "/.ssh")
    if not os.path.exists(configdir):
        os.makedirs(configdir)
    configdir = os.path.expanduser(MYHOME + "/.ssh/conf.d")
    if not os.path.exists(configdir):
        os.makedirs(configdir)
    configfilemaster = MYHOME + "/.ssh/config"
    if not os.path.isfile(configfilemaster):
        open(configfilemaster, "x").write("Include ~/.ssh/conf.d/*\n")
    includeexists = False
    with open(configfilemaster) as fh:
        for line in fh:
            if line.startswith("Include ~/.ssh/conf.d/*"):
                includeexists = True
    if not includeexists:
        open(configfilemaster, "w").write("Include ~/.ssh/conf.d/*\n")
    MYHOME = os.getenv("HOME")
    configfile = os.path.expanduser(MYHOME + "/.ssh/conf.d/ign8.conf")
    #create an empty file if its missing
    touch(configfile)
    touch(configfilemaster) 
    



    os.chmod(configfilemaster, 0o0600)
    os.chmod(configfile, 0o0600)
    # open config file and create it if its missing
    open(configfile, "w").write(ssh_config_content)
    prettyllog("ssh_config", "update ssh config", "~/.ssh/conf.d/organisation", "organization", "000", "new ssh config written", "INFO")




def touch(file_path):
    try:
        # Open the file in append mode, which creates the file if it doesn't exist
        with open(file_path, "a"):
            os.utime(file_path, None)  # Update the file's access and modification timestamps
        print(f"File '{file_path}' created or updated successfully.")
    except Exception as e:
        print(f"Error: {e}")


def generate_ssh_config_entry(vm):
    proxy_jump = ""
    if vm["cluster"] != vm["name"]:

        proxy_jump = f"ProxyJump {vm['cluster']}".split(".openknowit.com")[0]
        if proxy_jump == "None":
            return ssh_config_template.format(
            hostname=vm["name"],
            full_hostname=vm["name"] + ".openknowit.com",
            )
        else:
            return ssh_config_template.format(
            hostname=vm["name"],
            full_hostname=vm["name"] + ".openknowit.com",
            proxy_jump=proxy_jump
            )
    else:
        return ssh_config_template.format(
        hostname=vm["name"],
        full_hostname=vm["name"] + ".openknowit.com"
        )

import subprocess

def get_unique_sorted_items(filename):

# Run the 'cat inventory' command and capture its output
    cat_process = subprocess.Popen(['cat', 'inventory'], stdout=subprocess.PIPE)
    inventory_output, _ = cat_process.communicate()

# Split the output into lines, filter out lines starting with "[", and containing lowercase letters
    filtered_lines = [line for line in inventory_output.decode().splitlines() if not line.startswith('[') and any(c.islower() for c in line)]

# Extract the first column from each line
    first_columns = [line.split()[0] for line in filtered_lines]

# Remove duplicate entries and sort the result
    unique_sorted_items = sorted(set(first_columns))
    return unique_sorted_items

def serve():
    prettyllog("manage", "netbox", "serve", "000", "Starting netbox serve")
    prettyllog("manage", "netbox", "serve", "000", "Getting netbox token")
    global NETBOX_TOKEN
    NETBOX_TOKEN = os.environ.get('NETBOX_API_TOKEN')
    if NETBOX_TOKEN == None:
        prettyllog("manage", "netbox", "serve", "000", "No netbox token found")
        return False
    prettyllog("manage", "netbox", "serve", "000", "Getting netbox url")
    global NETBOX_URL
    NETBOX_URL = os.environ.get('NETBOX_API_URL')
    if NETBOX_URL == None:
        prettyllog("manage", "netbox", "serve", "000", "No netbox url found")
        return False
    prettyllog("manage", "netbox", "serve", "000", "Read the /etc/ign8/ign8_netbox.json file")
    data = {}
    with open('/etc/ign8/ign8_netbox.json') as json_file:
        data = json.load(json_file)
        prettyllog("manage", "netbox", "serve", "000", "The data is %s" % data)
        prettyllog("manage", "netbox", "serve", "000", "The data is %s" % data["netbox"])
        prettyllog("manage", "netbox", "serve", "000", "The data is %s" % data["netbox"]["token"])
        NETBOX_TOKEN = data["netbox"]["token"]
        NETBOX_URL = data["netbox"]["url"]
    
    

def inventory_upload():
    prettyllog("manage", "netbox", "inventory", "upload", "000", "Uploading inventory to netbox")
    prettyllog("manage", "netbox", "inventory", "upload", "000", "locating inventory file")
    inventory_file = os.environ.get("IGN8_INVENTORY_FILE")
    if inventory_file == None:
        inventory_file = "inventory"
    prettyllog("manage", "netbox", "inventory", "upload", "000", "inventory file is %s" % inventory_file)
    prettyllog("manage", "netbox", "inventory", "upload", "000", "print the inventory file")
    invfile = open(inventory_file, "r")
    print(invfile.read())
    prettyllog("manage", "netbox", "inventory", "upload", "000", "inventory file printed")
    items = get_unique_sorted_items(inventory_file)
    for item in items:
        prettyllog("manage", "netbox", "inventory", "upload", "000", "Adding %s to netbox" % item)
        os.environ["IGN8_DEVICE_NAME"] = item
        add_device()

def get_config():
    vmname = os.environ.get('HOSTNAME')
    if vmname == "localhost":
        return False
    # stript the name from the domain
    vmname = vmname.split(".")[0]
    prettyllog("manage", "netbox", "config", "get", "000", "Getting config for %s" % vmname)
    vmdata = get_virtual_machine(vmname)
    return vmdata

    







#!/usr/bin/env python
"""
Written by Chris Hupman
Github: https://github.com/chupman/
Example: Get guest info with folder and host placement

"""
import json
from tools import cli, service_instance


data = {}


def get_nics(guest):
    nics = {}
    for nic in guest.net:
        if nic.network:  # Only return adapter backed interfaces
            if nic.ipConfig is not None and nic.ipConfig.ipAddress is not None:
                nics[nic.macAddress] = {}  # Use mac as uniq ID for nic
                nics[nic.macAddress]['netlabel'] = nic.network
                ipconf = nic.ipConfig.ipAddress
                i = 0
                nics[nic.macAddress]['ipv4'] = {}
                for ip in ipconf:
                    if ":" not in ip.ipAddress:  # Only grab ipv4 addresses
                        nics[nic.macAddress]['ipv4'][i] = ip.ipAddress
                        nics[nic.macAddress]['prefix'] = ip.prefixLength
                        nics[nic.macAddress]['connected'] = nic.connected
                    i = i+1
    return nics


def vmsummary(summary, guest):
    vmsum = {}
    config = summary.config
    net = get_nics(guest)
    vmsum['mem'] = str(config.memorySizeMB / 1024)
    vmsum['diskGB'] = str("%.2f" % (summary.storage.committed / 1024**3))
    vmsum['cpu'] = str(config.numCpu)
    vmsum['path'] = config.vmPathName
    vmsum['ostype'] = config.guestFullName
    vmsum['state'] = summary.runtime.powerState
    vmsum['annotation'] = config.annotation if config.annotation else ''
    vmsum['net'] = net

    return vmsum


def vm2dict(datacenter, cluster, host, vm, summary):
    # If nested folder path is required, split into a separate function
    vmname = vm.summary.config.name
    data[datacenter][cluster][host][vmname]['folder'] = vm.parent.name
    data[datacenter][cluster][host][vmname]['mem'] = summary['mem']
    data[datacenter][cluster][host][vmname]['diskGB'] = summary['diskGB']
    data[datacenter][cluster][host][vmname]['cpu'] = summary['cpu']
    data[datacenter][cluster][host][vmname]['path'] = summary['path']
    data[datacenter][cluster][host][vmname]['net'] = summary['net']
    data[datacenter][cluster][host][vmname]['ostype'] = summary['ostype']
    data[datacenter][cluster][host][vmname]['state'] = summary['state']
    data[datacenter][cluster][host][vmname]['annotation'] = summary['annotation']


def data2json(raw_data, args):
    with open(args.jsonfile, 'w') as json_file:
        json.dump(raw_data, json_file)


def main():
    """
    Iterate through all datacenters and list VM info.
    """
    parser = cli.Parser()
    parser.add_custom_argument('--json', required=False, action='store_true',
                               help='Write out to json file')
    parser.add_custom_argument('--jsonfile', required=False, action='store',
                               default='getvmsbycluster.json',
                               help='Filename and path of json file')
    parser.add_custom_argument('--silent', required=False, action='store_true',
                               help='supress output to screen')
    args = parser.get_args()
    si = service_instance.connect(args)
    outputjson = True if args.json else False

    content = si.RetrieveContent()
    children = content.rootFolder.childEntity
    for child in children:  # Iterate though DataCenters
        datacenter = child
        data[datacenter.name] = {}  # Add data Centers to data dict
        clusters = datacenter.hostFolder.childEntity
        for cluster in clusters:  # Iterate through the clusters in the DC
            # Add Clusters to data dict
            data[datacenter.name][cluster.name] = {}
            hosts = cluster.host  # Variable to make pep8 compliance
            for host in hosts:  # Iterate through Hosts in the Cluster
                hostname = host.summary.config.name
                # Add VMs to data dict by config name
                data[datacenter.name][cluster.name][hostname] = {}
                vms = host.vm
                for vm in vms:  # Iterate through each VM on the host
                    vmname = vm.summary.config.name
                    data[datacenter.name][cluster.name][hostname][vmname] = {}
                    summary = vmsummary(vm.summary, vm.guest)
                    vm2dict(datacenter.name, cluster.name, hostname, vm, summary)

    if not args.silent:
        print(json.dumps(data, sort_keys=True, indent=4))

    if outputjson:
        data2json(data, args)


# Start program
if __name__ == "__main__":
    main()



# VMware vSphere Python SDK
# Copyright (c) 2008-2021 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Utility functions for the vSphere API

See com.vmware.apputils.vim25.ServiceUtil in the java API.
"""

from pyVmomi import vim, vmodl


def build_full_traversal():
    """
    Builds a traversal spec that will recurse through all objects .. or at
    least I think it does. additions welcome.

    See com.vmware.apputils.vim25.ServiceUtil.buildFullTraversal in the java
    API. Extended by Sebastian Tello's examples from pysphere to reach networks
    and datastores.
    """

    traversal_spec = vmodl.query.PropertyCollector.TraversalSpec
    selection_spec = vmodl.query.PropertyCollector.SelectionSpec

    # Recurse through all resourcepools
    rp_to_rp = traversal_spec(name='rpToRp', type=vim.ResourcePool, path="resourcePool", skip=False)

    rp_to_rp.selectSet.extend(
        (
            selection_spec(name="rpToRp"),
            selection_spec(name="rpToVm"),
        )
    )

    rp_to_vm = traversal_spec(name='rpToVm', type=vim.ResourcePool, path="vm", skip=False)

    # Traversal through resourcepool branch
    cr_to_rp = traversal_spec(
        name='crToRp', type=vim.ComputeResource, path='resourcePool', skip=False)
    cr_to_rp.selectSet.extend(
        (
            selection_spec(name='rpToRp'),
            selection_spec(name='rpToVm'),
        )
    )

    # Traversal through host branch
    cr_to_h = traversal_spec(name='crToH', type=vim.ComputeResource, path='host', skip=False)

    # Traversal through hostFolder branch
    dc_to_hf = traversal_spec(name='dcToHf', type=vim.Datacenter, path='hostFolder', skip=False)
    dc_to_hf.selectSet.extend(
        (
            selection_spec(name='visitFolders'),
        )
    )

    # Traversal through vmFolder branch
    dc_to_vmf = traversal_spec(name='dcToVmf', type=vim.Datacenter, path='vmFolder', skip=False)
    dc_to_vmf.selectSet.extend(
        (
            selection_spec(name='visitFolders'),
        )
    )

    # Traversal through network folder branch
    dc_to_net = traversal_spec(
        name='dcToNet', type=vim.Datacenter, path='networkFolder', skip=False)
    dc_to_net.selectSet.extend(
        (
            selection_spec(name='visitFolders'),
        )
    )

    # Traversal through datastore branch
    dc_to_ds = traversal_spec(name='dcToDs', type=vim.Datacenter, path='datastore', skip=False)
    dc_to_ds.selectSet.extend(
        (
            selection_spec(name='visitFolders'),
        )
    )

    # Recurse through all hosts
    h_to_vm = traversal_spec(name='hToVm', type=vim.HostSystem, path='vm', skip=False)
    h_to_vm.selectSet.extend(
        (
            selection_spec(name='visitFolders'),
        )
    )

    # Recurse through the folders
    visit_folders = traversal_spec(
        name='visitFolders', type=vim.Folder, path='childEntity', skip=False)
    visit_folders.selectSet.extend(
        (
            selection_spec(name='visitFolders'),
            selection_spec(name='dcToHf'),
            selection_spec(name='dcToVmf'),
            selection_spec(name='dcToNet'),
            selection_spec(name='crToH'),
            selection_spec(name='crToRp'),
            selection_spec(name='dcToDs'),
            selection_spec(name='hToVm'),
            selection_spec(name='rpToVm'),
        )
    )

    full_traversal = selection_spec.Array(
        (visit_folders, dc_to_hf, dc_to_vmf, dc_to_net, cr_to_h, cr_to_rp, dc_to_ds, rp_to_rp,
         h_to_vm, rp_to_vm,))

    return full_traversal


# vim: set ts=4 sw=4 expandtab filetype=python:



# selinux/serializers.py

from rest_framework import serializers
from .models import Selinux, SElinuxEvent


class SElinuxEventSerializer(serializers.ModelSerializer):
    class Meta:
        model = SElinuxEvent
        fields = ['digest', 'hostname', 'event', 'date', 'time', 'serial_num', 'event_kind', 'session', 'subj_prime', 'subj_sec', 'subj_kind', 'action', 'result', 'obj_prime', 'obj_sec', 'obj_kind', 'how']

class SelinuxDataSerializer(serializers.ModelSerializer):
    class Meta:
        model = Selinux
        fields = ['hostname', 'status', 'mount', 'rootdir', 'policyname', 'current_mode', 'configured_mode', 'mslstatus', 'memprotect', 'maxkernel', 'total', 'success', 'failed', 'sealerts']





"""
Written by Michael Rice
Github: https://github.com/michaelrice
Website: https://michaelrice.github.io/
Blog: http://www.errr-online.com/
This code has been released under the terms of the Apache 2.0 license
http://opensource.org/licenses/Apache-2.0

This sample can be run directly to create a datacenter, but may be more
useful to import into another project where you need to create a datacenter
then use that object to further configure things like create a cluster or
adding resources like HostSystems.
"""
from pyVmomi import vim


def create_datacenter(dc_name=None, service_instance=None, folder=None):
    """
    Creates a new datacenter with the given name.
    Any % (percent) character used in this name parameter must be escaped,
    unless it is used to start an escape sequence. Clients may also escape
    any other characters in this name parameter.

    An entity name must be a non-empty string of
    less than 80 characters. The slash (/), backslash (\\) and percent (%)
    will be escaped using the URL syntax. For example, %2F

    This can raise the following exceptions:
    vim.fault.DuplicateName
    vim.fault.InvalidName
    vmodl.fault.NotSupported
    vmodl.fault.RuntimeFault
    ValueError raised if the name len is > 79
    https://github.com/vmware/pyvmomi/blob/master/docs/vim/Folder.rst

    Required Privileges
    Datacenter.Create

    :param folder: Folder object to create DC in. If None it will default to
                   rootFolder
    :param dc_name: Name for the new datacenter.
    :param service_instance: ServiceInstance connection to a given vCenter
    :return:
    """
    if len(dc_name) > 79:
        raise ValueError("The name of the datacenter must be under "
                         "80 characters.")
    if folder is None:
        folder = service_instance.content.rootFolder

    if folder is not None and isinstance(folder, vim.Folder):
        dc_moref = folder.CreateDatacenter(name=dc_name)
        return dc_moref


if __name__ == "__main__":
    import atexit
    from pyVim import connect
    import cli
    PARSER = cli.build_arg_parser()
    PARSER.add_argument("-n", "--name",
                        required=True,
                        action="store",
                        help="Name of the Datacenter to create.")
    MY_ARGS = PARSER.parse_args()
    cli.prompt_for_password(MY_ARGS)
    SI = connect.SmartConnect(host=MY_ARGS.host,
                              user=MY_ARGS.user,
                              pwd=MY_ARGS.password,
                              port=MY_ARGS.port)
    create_datacenter(dc_name=MY_ARGS.name, service_instance=SI)
    atexit.register(connect.Disconnect, SI)



from PIL import Image, ImageDraw

# Create a list to store frames
frames = []

# Create frames
for i in range(10):
    print("Creating frame", i)
    # Create a new image
    image = Image.new("RGB", (200, 200), "white")
    
    # Draw something on the image (e.g., a rectangle)
    draw = ImageDraw.Draw(image)
    draw.rectangle([i * 20, 50, i * 20 + 30, 150], fill="blue")

    # Append the frame to the list
    frames.append(image)

# Save the frames as an animated GIF
frames[0].save("animated.gif", save_all=True, append_images=frames[1:], duration=100, loop=0)




"""
Written by Michael Rice <michael@michaelrice.org>

Github: https://github.com/michaelrice
Website: https://michaelrice.github.io/
Blog: http://www.errr-online.com/
This code has been released under the terms of the Apache 2 licenses
http://www.apache.org/licenses/LICENSE-2.0.html

Helper module for task operations.
"""
from pyVmomi import vim
from pyVmomi import vmodl


def wait_for_tasks(si, tasks):
    """Given the service instance and tasks, it returns after all the
   tasks are complete
   """
    property_collector = si.content.propertyCollector
    task_list = [str(task) for task in tasks]
    # Create filter
    obj_specs = [vmodl.query.PropertyCollector.ObjectSpec(obj=task)
                 for task in tasks]
    property_spec = vmodl.query.PropertyCollector.PropertySpec(type=vim.Task,
                                                               pathSet=[],
                                                               all=True)
    filter_spec = vmodl.query.PropertyCollector.FilterSpec()
    filter_spec.objectSet = obj_specs
    filter_spec.propSet = [property_spec]
    pcfilter = property_collector.CreateFilter(filter_spec, True)
    try:
        version, state = None, None
        # Loop looking for updates till the state moves to a completed state.
        while task_list:
            update = property_collector.WaitForUpdates(version)
            for filter_set in update.filterSet:
                for obj_set in filter_set.objectSet:
                    task = obj_set.obj
                    for change in obj_set.changeSet:
                        if change.name == 'info':
                            state = change.val.state
                        elif change.name == 'info.state':
                            state = change.val
                        else:
                            continue

                        if not str(task) in task_list:
                            continue

                        if state == vim.TaskInfo.State.success:
                            # Remove task from taskList
                            task_list.remove(str(task))
                        elif state == vim.TaskInfo.State.error:
                            raise task.info.error
            # Move to next version
            version = update.version
    finally:
        if pcfilter:
            pcfilter.Destroy()



# Generated by Django 4.2.8 on 2023-12-22 21:51

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Selinux',
            fields=[
                ('hostname', models.CharField(max_length=128, primary_key=True, serialize=False)),
                ('status', models.CharField(max_length=50)),
                ('mount', models.CharField(max_length=50)),
                ('rootdir', models.CharField(max_length=50)),
                ('policyname', models.CharField(max_length=50)),
                ('current_mode', models.CharField(max_length=50)),
                ('configured_mode', models.CharField(max_length=50)),
                ('mslstatus', models.CharField(max_length=50)),
                ('memprotect', models.CharField(max_length=50)),
                ('maxkernel', models.CharField(max_length=50)),
                ('total', models.CharField(max_length=50)),
                ('success', models.CharField(max_length=50)),
                ('failed', models.CharField(max_length=50)),
                ('sealerts', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Selinux',
                'verbose_name_plural': 'Selinux',
                'db_table': 'selinux',
                'ordering': ['hostname'],
            },
        ),
        migrations.CreateModel(
            name='SElinuxEvent',
            fields=[
                ('digest', models.CharField(max_length=256, primary_key=True, serialize=False)),
                ('hostname', models.CharField(max_length=128)),
                ('event', models.CharField(max_length=1024)),
                ('date', models.DateField()),
                ('time', models.TimeField()),
                ('serial_num', models.IntegerField()),
                ('event_kind', models.CharField(blank=True, max_length=256, null=True)),
                ('session', models.CharField(blank=True, max_length=256, null=True)),
                ('subj_prime', models.CharField(blank=True, max_length=256, null=True)),
                ('subj_sec', models.CharField(blank=True, max_length=256, null=True)),
                ('subj_kind', models.CharField(blank=True, max_length=256, null=True)),
                ('action', models.CharField(blank=True, max_length=256, null=True)),
                ('result', models.CharField(blank=True, max_length=256, null=True)),
                ('obj_prime', models.CharField(blank=True, max_length=256, null=True)),
                ('obj_sec', models.CharField(blank=True, max_length=256, null=True)),
                ('obj_kind', models.CharField(blank=True, max_length=256, null=True)),
                ('how', models.CharField(blank=True, max_length=256, null=True)),
            ],
            options={
                'verbose_name': 'SElinuxEvent',
                'verbose_name_plural': 'SElinuxEvent',
                'db_table': 'selinux_event',
                'ordering': ['date', 'time', 'hostname'],
            },
        ),
    ]



from . import traefik
import argparse

def main():
    parser = argparse.ArgumentParser(description="Keep ign8 and automate", usage="ign8_traefik <action> \n\n \
               \
               version : 0.1.2 traefik  \n                                              \
               actions:\n                                                      \
               list        list traefik \n  \
               set         set traefik times \n  \
               plugins     get traefik plugins \n  \
               \
               2023 Knowit Miracle\
               ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup jenkis')
    args = parser.parse_args()
    ready = False
    print("check if we are ready to go")

    if args.action[0] == "list":
        print("list traefik")
        traefik.list_traefik()

    if args.action[0] == "serve":
        traefik.serve_traefik()
        




import requests
import json
import os
import base64
import xml.etree.ElementTree as ET
import pprint
from ..common import prettyllog


import base64


def getenv():
  myenv = {}
  myenv["IGN8_GIT_URL"] = os.getenv("IGN8_GIT_URL")
  myenv["IGN8_GIT_USER"] = os.getenv("IGN8_GIT_USER")
  myenv["IGN8_GIT_PASSWORD"] = os.getenv("IGN8_GIT_PASSWORD")
  myenv["IGN8_GIT_TYPE"] = os.getenv("IGN8_GIT_TYPE")
  username = os.getenv("IGN8_GIT_USER")
  password = os.getenv("IGN8_GIT_PASSWORD")
  myenv["verifyssl"] = os.getenv("IGN8_GIT_VERIFY_SSL", "False")

  credentials = f"{username}:{password}"
  base64_credentials = base64.b64encode(credentials.encode("utf-8")).decode("utf-8")
  myenv["base64_credentials"] = base64_credentials
  return myenv

def init():
  prettyllog("state", "Init", "git", "start", "000", "login initiated", severity="DEBUG")
  myenv = getenv()
  session = requests.Session()
  url = os.getenv("IGN8_GIT_URL") + "/api/v1/user"
  headers = {
    "Content-Type": "application/json",
    "Authorization": "Basic " + myenv['base64_credentials']
    }
  resp = session.get(url,headers=headers)
  if resp.status_code == 200:
    prettyllog("state", "Init", "git", "ok", resp.status_code, "login successful", severity="INFO")
    return session
  else:
    prettyllog("state", "Init", "git", "error", resp.status_code, "login failed", severity="ERROR")
    return None
def create_git_project(project):
  prettyllog("state", "Init", "git", "start", "000", "create project initiated", severity="DEBUG")
  myenv = getenv()
  session = init()
  url = myenv['IGN8_GIT_URL'] + "/api/v1/user/repos"
  headers = {
    "Content-Type": "application/json",
    "Authorization": "Basic " + myenv['base64_credentials']
    }
  data = {
    "name": project['name'],
    "description": project['description'],
    "private": project['private'],
    "auto_init": project['auto_init']
    }
  resp = session.post(url,headers=headers, json=data)
  if resp.status_code == 201:
    prettyllog("state", "Init", "git", "ok", resp.status_code, "create project successful", severity="INFO")
    return resp.json()
  else:
    prettyllog("state", "Init", "git", "error", resp.status_code, "create project failed", severity="ERROR")
    return None
  
  
def get_git_projects():
  myenv = getenv() 
  session = init()
  url = myenv['IGN8_GIT_URL'] + "/api/v1/user/repos"
  headers = {
    "Content-Type": "application/json",
    "Authorization": "Basic " + myenv['base64_credentials']
    }
  resp = session.get(url,headers=headers)
  if resp.status_code == 200:
    prettyllog("state", "Init", "git", "ok", resp.status_code, "get projects successful", severity="INFO")
    return resp.json()
  else:
    prettyllog("state", "Init", "git", "error", resp.status_code, "get projects failed", severity="ERROR")
    return None
  

  
def create_git_token():
  session = init()
  myenv = getenv()
  url = myenv['IGN8_GIT_URL'] + "/api/v1/users/" + myenv['IGN8_GIT_USER'] + "/tokens?sudo=" + myenv['IGN8_GIT_USER']
  headers = {
    "Content-Type": "application/json",
    "Authorization": "Basic " + myenv['base64_credentials']
    }
  data = {
    "name": myenv['IGN8_GIT_USER']
    }
  resp = session.post(url,headers=headers, json=data)
  if (resp.status_code == 201):
    prettyllog("state", "Init", "git", "ok", resp.status_code, "create token successful", severity="INFO")
    return resp.json()
  else:
    prettyllog("state", "Init", "git", "error", resp.status_code, "create token failed", severity="INFO")
    return None



def refresh_git_token(git_token):
  create = True
  if (len (git_token) == 0):
    prettyllog("state", "Init", "git", "error", "000", "no tokens found", severity="INFO")
    create = Truesyst
    mytoken = create_git_token()
    pprint.pprint(mytoken)
  else:
    prettyllog("state", "Init", "git", "ok", "000", "tokens found", severity="INFO")
    mytoken = create_git_token()
    pprint.pprint(mytoken)
  if create == True:
    mytoken = create_git_token()



def delete_token():
  prettyllog("state", "Init", "git", "ok", "000", "token found", severity="INFO")
  session = init()
  myenv = getenv()
  url = myenv['IGN8_GIT_URL'] + "/api/v1/users/" + myenv['IGN8_GIT_USER'] + "/tokens/"
  headers = {
    "Content-Type": "application/json",
    "Authorization": "Basic " + myenv['base64_credentials']
    }
  data = {
    "username": myenv['IGN8_GIT_USER']
    }
  resp = session.delete(url,headers=headers)
  if (resp.status_code == 204):
    prettyllog("state", "Init", "git", "ok", resp.status_code, "delete token successful", severity="INFO")
  else:
    prettyllog("state", "Init", "git", "error", resp.status_code, "delete token failed", severity="INFO")
  return None


def get_git_tokens():

  session = init()
  myenv = getenv()
  url = myenv['IGN8_GIT_URL'] + "/api/v1/users/" + myenv['IGN8_GIT_USER'] + "/tokens?sudo=" + myenv['IGN8_GIT_USER']
  headers = {
    "Content-Type": "application/json",
    "Authorization": "Basic " + myenv['base64_credentials']
    }
  data = {
    "username": myenv['IGN8_GIT_USER']
    }
  resp = session.get(url,headers=headers, json=data)
  if (resp.status_code == 200):
    prettyllog("state", "Init", "git", "ok", resp.status_code, "get token successful", severity="INFO")
    return resp.json()
  else:
    prettyllog("state", "Init", "git", "error", resp.status_code, "get token failed", severity="INFO")
    return None
  


 



import os
import sys
import redis
import time
import subprocess
import json
import argparse
import requests
from .common import prettyllog
from .main import main
from .setup import setupign8






class suppress_stdout_stderr(object):
    '''
    A context manager for doing a "deep suppression" of stdout and stderr in 
    Python, i.e. will suppress all print, even if the print originates in a 
    compiled C/Fortran sub-function.
       This will not suppress raised exceptions, since exceptions are printed
    to stderr just before a script exits, and after the context manager has
    exited (at least, I think that is why it lets exceptions through).      

    '''
    def __init__(self):
        # Open a pair of null files
        self.null_fds =  [os.open(os.devnull,os.O_RDWR) for x in range(2)]
        # Save the actual stdout (1) and stderr (2) file descriptors.
        self.save_fds = [os.dup(1), os.dup(2)]

    def __enter__(self):
        # Assign the null pointers to stdout and stderr.
        os.dup2(self.null_fds[0],1)
        os.dup2(self.null_fds[1],2)

    def __exit__(self, *_):
        # Re-assign the real stdout/stderr back to (1) and (2)
        os.dup2(self.save_fds[0],1)
        os.dup2(self.save_fds[1],2)
        # Close all file descriptors
        for fd in self.null_fds + self.save_fds:
            os.close(fd)



def runme(command):
  commandlist = command.split(" ")
  result = subprocess.run(commandlist, capture_output=True)
  payload = {"returncode": result.returncode, "stdout": result.stdout.decode(), "stderr": result.stderr }
  return payload


def connectiontest():
  checks=[]
  errors=[]
  if len(errors) > 0:
    prettyllog("connectiontest", "check", "connectiontest", "all", "500", "Failed", "ERROR")
    return False
  else:
    return True
    
def main():
    usage =  "Usage:"
    usage += " ign8 <action> \n\n"
    usage += "Actions:\n"
    usage += "           check\n"
    usage += "           setup\n"
    usage += "           reset\n"
    usage += "           startservice\n"
    usage += "           stopservice\n"
    usage += "           initservice\n"
    usage += "           service\n\n"
    usage += "           2024 ign8.it "
    parser = argparse.ArgumentParser(description="Keep ign8 and automate", usage=usage)
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup netbox')
    args = parser.parse_args()
    ready = False
    ready  = connectiontest()

    if args.action[0] == "setup":
        setupign8()
    elif args.action[0] == "check":
        print("Checking")
    elif args.action[0] == "reset":
        print("Resetting")
    elif args.action[0] == "startservice":
        print("Starting Service")
    elif args.action[0] == "stopservice":
        print("Stopping Service")
    elif args.action[0] == "initservice":
        print("Initializing Service")
    elif args.action[0] == "service":
        print("Service")

    








import os
import datetime
import subprocess 


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    DEBUG   = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    INFO    = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


VERIFY_SSL = os.getenv("VERIFY_SSL", "false")
if VERIFY_SSL == "false" or VERIFY_SSL == "False" or VERIFY_SSL == "FALSE" or VERIFY_SSL == "no" or VERIFY_SSL == "NO" or VERIFY_SSL == "No":
  VERIFY_SSL = False
else:
  VERIFY_SSL = True

def line_in_file(file_path, search_text):
    with open(file_path, 'r') as f:
        for line in f:
            if search_text in line:
                return True
    return False
def runme(command):
  commandlist = command.split(" ")
  result = subprocess.run(commandlist, capture_output=True)
  payload = {"returncode": result.returncode, "stdout": result.stdout.decode(), "stderr": result.stderr }
  return payload

    

def get_file_content(file_path):
    with open(file_path, 'r') as f:
        return f.read()

def get_file_content_lines(file_path):
    with open(file_path, 'r') as f:
        return f.readlines()
    
# 127.0.0.1 user-identifier frank [10/Oct/2000:13:55:36 -0700] "GET /apache_pb.gif HTTP/1.0" 200 2326
    #    127.0.0.1 is the IP address of the client (remote host) which made the request to the server.
    # user-identifier is the RFC 1413 identity of the client. Usually "-".
    #frank is the userid of the person requesting the document. Usually "-" unless .htaccess has requested authentication.
    #[10/Oct/2000:13:55:36 -0700] is the date, time, and time zone that the request was received, by default in strftime format %d/%b/%Y:%H:%M:%S %z.
    #"GET /apache_pb.gif HTTP/1.0" is the request line from the client. The method GET, /apache_pb.gif the resource requested, and HTTP/1.0 the HTTP protocol.
    #200 is the HTTP status code returned to the client. 2xx is a successful response, 3xx a redirection, 4xx a client error, and 5xx a server error.
    #2326 is the size of the object returned to the client, measured in bytes.
# we need to rewrite prettylog to use this format
    

    

def prettyllog(function, action, item, organization, statuscode, text, severity="INFO"):
  silence = False
  try:
    if os.getenv("IGN8_SILENCE", "false").lower() == "true":
      silence = True
  except:
    silence = False
    
  if silence:
    return True

  d_date = datetime.datetime.now()
  reg_format_date = d_date.strftime("%Y-%m-%d %I:%M:%S %p")
  if severity == "INFO":
    print(f"{bcolors.INFO}%-20s: %-12s %20s %-50s %-20s %-4s %-50s " %( reg_format_date, function,action,item,organization,statuscode, text))
  elif severity == "WARNING":
    print(f"{bcolors.WARNING}%-20s: %-12s %20s %-50s %-20s %-4s %-50s " %( reg_format_date, function,action,item,organization,statuscode, text))
  elif severity == "ERROR":
    print(f"{bcolors.FAIL}%-20s: %-12s %20s %-50s %-20s %-4s %-50s " %( reg_format_date, function,action,item,organization,statuscode, text))
  elif severity == "DEBUG":
    print(f"{bcolors.OKCYAN}%-20s: %-12s %20s %-50s %-20s %-4s %-50s " %( reg_format_date, function,action,item,organization,statuscode, text))
  else:
    print(f"{bcolors.INFO}%-20s: %-12s %20s %-50s %-20s %-4s %-50s " %( reg_format_date, function,action,item,organization,statuscode, text))
  print(f"{bcolors.ENDC}", end='')
  return True



import argparse
import os


from ..common import prettyllog
from . import serve
from . import stop
from . import start
from . import status
from . import selinux
from . import firewall
from . import setup


def main():
    parser = argparse.ArgumentParser(description="Keep ign8 and automate", usage="ign8_ui <action> \n\n\
\
                version : 0.0.1 \n\
                actions:\n\
                serve                        run the service as in systemd but with stdout on the terminal\n\
                start                        start the service in systemd\n\
                stop                         stop the systemd\n\
                status                       Show the status of the systemd services\n\
                selinux                      Ensure selinux is configured\n\
                firewall                     Ensure Firewall is configured\n\
                setup                        Setup server to host ign8_ui\n\
\
                2023 Knowit Miracle\n\
                ")
    parser.add_argument('action', metavar='<action>', type=str, nargs='+', help='setup netbox')
    args = parser.parse_args()
    ready = False
    if args.action[0] == "serve":
        prettyllog("ui", "ui", "ui", "new", "000", "serve")
        serve.main()
        return 0
    
    if args.action[0] == "start":
        prettyllog("ui", "ui", "ui", "new", "000", "start")
        start.main()
        return 0
    
    if args.action[0] == "stop":
        prettyllog("ui", "ui", "ui", "new", "000", "stop")
        stop.main()
        return 0
    
    if args.action[0] == "status":
        prettyllog("ui", "ui", "ui", "new", "000", "status")
        status.main()
        return 0
    
    if args.action[0] == "selinux":
        prettyllog("ui", "ui", "ui", "new", "000", "selinux")
        selinux.main()
        return 0
    
    if args.action[0] == "firewall":
        prettyllog("ui", "ui", "ui", "new", "000", "firewall")
        firewall.main()
        return 0
    
    if args.action[0] == "setup":
        prettyllog("ui", "ui", "ui", "new", "000", "setup")
        setup.main()
        return 0
    
    

        
